/*
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//
// @author Marco Magistrali
//

#include <windows.h>

#include "pimstore.h"
#include <stdio.h>

#include "common/Log.h"
#include "spds/common/Utils.h"
#include "spds/common/Constants.h"
#include "spds/common/SyncManagerFactory.h"

#include "pim/ActionLog.h"
#include "pim/Utils.h"

#include "pim/FILEServer2Client.h"
#include "pim/FILEFileManagement.h"

void manageReturnedItemsFILE(SyncSource& s, int dataType, wchar_t* path, wchar_t* dir) {
    unsigned int n, i, j = 0;
    SyncItem** items;       
    SyncItem** updatedSyncItems;

    wchar_t localTemp [DIM_LOCAL_TEMP];

    ActionLOG.action(MESSAGE_FROM_SERVER);

    //UPDATE************
        
    n = s.getUpdatedSyncItemsCount();
    updatedSyncItems = s.getUpdatedSyncItems();
        
    wsprintf(localTemp,SERVER_UPDATE_ITEMS, n );
    ActionLOG.action(localTemp);

    if (dbg) {
        wsprintf(localTemp, TEXT("update item are: %i"), n );
        MessageBox (NULL, localTemp, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
    }

    int* elementIndexToInsert = NULL;

    if (n > 0) {
            
        elementIndexToInsert = new int[n];      
                        
        for (i=0; i<n; ++i) {
            elementIndexToInsert[i] = NULL; // every oidToInsert pointer initialized to null!
        }
    }

    LOG.info(TEXT("Updated items"));
    LOG.info(TEXT("============="));
    for (i = 0; i < n; ++i) {                               
      
        if (manageUpdatedItems(updatedSyncItems[i], dataType, path, dir) != 0) {
            elementIndexToInsert[j] = i;
            j++;
        }
    }

//DELETE*******************

    n = s.getDeletedSyncItemsCount();
    items = s.getDeletedSyncItems();    

    wsprintf(localTemp,SERVER_DELETE_ITEMS, n );
    ActionLOG.action(localTemp);

    if (dbg) {
        wsprintf(localTemp, TEXT("delete item are: %i"), n );
        MessageBox (NULL, localTemp, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
    }

    LOG.info(TEXT("Deleted items"));
    LOG.info(TEXT("============="));
    for (i=0; i<n; ++i) {

        manageDeletedItems(items[i]->getKey(NULL), dataType, path, dir);
    }    



//NEW *********************
// ------------------------------------------------------------------------------------
        
    n = s.getNewSyncItemsCount();
    items = s.getNewSyncItems();
        
    wsprintf(localTemp,SERVER_NEW_ITEMS, n );
    ActionLOG.action(localTemp);

    // array in which store the new oid for the mapping...
    // delete in the end every pointer...
    if (dbg) {      
        wsprintf(localTemp, TEXT("Return new item are: %i"), n );
        MessageBox (NULL, localTemp, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
    }
    
    wchar_t **ptrLuid = NULL;
    wchar_t luid [DIM_KEY];
    SyncMap**  mappings = NULL;
    
    // da fare solo se n > 0!!!
    if (n + j> 0) {                         
    
        mappings = new SyncMap* [n + j];
        for (i = 0; i < n + j; ++i) {
            mappings[i] = NULL; // every map pointer initialized to null!
        }
    
        ptrLuid = new wchar_t*[n + j];
        for (i = 0; i < n + j ; ++i) {
            ptrLuid[i] = NULL; // every map pointer initialized to null!
        }

    }
    
    LOG.info(TEXT("New items"));
    LOG.info(TEXT("========="));
    string luid_s = string();  // used to encode specialChar (&,<,>)
    for (i=0; i < n; ++i) {
                
        ptrLuid[i] = manageNewItems(items[i], dataType, path, dir);
        wsprintf(luid, TEXT("%s"), ptrLuid[i] );
        
        if (dbg) {
            MessageBox (NULL, luid, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
                    
            wsprintf(localTemp, TEXT("chiave: %s"), items[i]->getKey(NULL) );
            MessageBox (NULL, localTemp, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);

            wsprintf(localTemp, TEXT("luid: %s"), luid );
            MessageBox (NULL, localTemp, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
        }
        luid_s = string(luid);
        encodeSpecialChar(luid_s);
        mappings[i] = new SyncMap(items[i]->getKey(NULL), (wchar_t*)luid_s);
        // mappings[i] = new SyncMap(items[i]->getKey(NULL), luid);
    }
    unsigned int k = 0;
    for (i = n; i < n + j; ++i) {
        
        ptrLuid[i] = manageNewItems(updatedSyncItems[elementIndexToInsert[k]], dataType, path, dir);
        wsprintf(luid, TEXT("%s"), ptrLuid[i] );
        
        if (dbg) {
            MessageBox (NULL, luid, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
                    
            wsprintf(localTemp, TEXT("chiave: %s"), updatedSyncItems[elementIndexToInsert[k]]->getKey(NULL) );
            MessageBox (NULL, localTemp, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);

            wsprintf(localTemp, TEXT("luid: %s"), luid );
            MessageBox (NULL, localTemp, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
        }
        luid_s = string(luid);
        encodeSpecialChar(luid_s);
        mappings[i] = new SyncMap(updatedSyncItems[elementIndexToInsert[k]]->getKey(NULL), (wchar_t*)luid_s);
        // mappings[i] = new SyncMap(updatedSyncItems[elementIndexToInsert[k]]->getKey(NULL), luid);
        k++;
        
    }

            
    // mapping for the new element if and only if i > 0
    if (n + j > 0) {                
        s.setLUIDGUIDMapping(mappings, n + j);          
    }
    
    for (k = 0; k < n + j; k++) {
        if (ptrLuid[k] != NULL)
            delete ptrLuid[k]; 
    }    
    if (ptrLuid != NULL)
        delete [] ptrLuid;

   
// ------------------------------------------------------------------------------------_
   
    // write current fileName and date of modify into file      
    writeCurrentFileItems(dataType, path, dir);
   
}

/*
* Translate base64 data to text...
*/
void translateBase642Text(wchar_t** ptrData) {
    
    wchar_t* ret = NULL;
    ret = decodeBase64WUTF8(*ptrData);
    if (*ptrData != NULL) {
        delete *ptrData; 
        *ptrData = NULL;
    }
    
    *ptrData = ret;
    
    /*
    int num = 0;
    char* content = wchar2char(*ptrData);    
    if (content == NULL) {        
        return;
    }
    
    byte* reversed = new byte[strlen(content) + 1];   
    memset(reversed, 0, strlen(content) + 1);
    decodeBase64(content, reversed, &num);
    reversed[num] = 0;

    delete *ptrData; *ptrData = NULL;
    delete content; content = NULL;

    *ptrData = char2wchar((char*)reversed);
    
    delete reversed; reversed = NULL;
    */
}

/*
* Translate base64 data to text with only one pointer...
*/
wchar_t* translateBase642Text_2(wchar_t* ptrData) {

    int num = 0;
    char* content = wchar2char(ptrData);
    
    if (content == NULL) {        
        return ptrData;
    }

    if (dbg)
        MessageBox (NULL, ptrData, TEXT ("ptrData"), MB_SETFOREGROUND |MB_OK);

    byte* reversed = new byte[strlen(content) + 1];   
    memset(reversed, 0, strlen(content) + 1);
    decodeBase64(content, reversed, &num);
    reversed[num] = 0;

    if (dbg) {
        wchar_t ts [256];
        wsprintf(ts, TEXT("-%i-"), num);
        MessageBox (NULL, ts, TEXT ("num"), MB_SETFOREGROUND |MB_OK);
    }
    
    delete ptrData; ptrData = NULL;
    delete content; content = NULL;

    ptrData = char2wchar((char*)reversed);
    
    delete reversed; reversed = NULL;
    return ptrData;
}


/*
* method used only for notes and not for files. Files cannot be modified...
*/
long manageUpdatedItems(SyncItem* item, int dataType, wchar_t* path, wchar_t* dir) {

    LOG.info(TEXT("in managerUpdatedItems ... "));                  

    wchar_t localTemp   [DIM_LOCAL_TEMP];    
    wchar_t tmp         [DIM_LOCAL_TEMP];
    wchar_t* ptrData    = NULL;    
    
    int size = 0;
    size = wcslen((wchar_t*) item->getData()) + 1;
    ptrData = new wchar_t [size];    
    wcscpy(ptrData, (wchar_t*) item->getData());
    ptrData[size] = 0;
    // used to remove start CarriageReturn into return data stream.
    // to be remove when correct into api methods. 
    // 2004-07-08 fixed in SC-API
    removeStartCarriage(&ptrData);    

    // ptrData = (wchar_t*) item->getData();    
    DWORD   result = 0;    
    
    wsprintf(tmp, TEXT("%s"), (wchar_t*)item->getKey(NULL));    
    decodeKey (tmp);
    
    // dir from server arrives with \ at the end. It must be removed

    if ( tmp[wcslen(tmp) - 1] == L'\\')
        tmp[wcslen(tmp) - 1] = 0;
    
    if (dbg) 
        MessageBox (NULL, tmp, TEXT ("tmp"), MB_SETFOREGROUND |MB_OK);
        
    if (dataType == constFiles) {
        //
        // nothing to do. It doesn't happen. on server files cannot change...
        //
    }
    else if (dataType == constNotes) {
    
        wsprintf(localTemp, TEXT("%s%s%s"), dir, TEXT("\\"), tmp);
        if (dbg) 
            MessageBox (NULL, localTemp, TEXT ("path and file Note"), MB_SETFOREGROUND |MB_OK);
    
        // if file doesn't exist. ---------
        result = GetFileAttributes(localTemp);

        if (result == 0xFFFFFFFF || result == -1) {
        
            if (ptrData != NULL) {
                delete ptrData;
                ptrData = NULL;
            }
            
            return -1;

        }
        // -------------------   

        // translate base64 into text
        if (ptrData != NULL) {      
            translateBase642Text(&ptrData);
            // ptrData = translateBase642Text_2(ptrData);
        }
    
        // parsing of xml structure to retrieve subject as filename and body as text.
        wchar_t* body = NULL;
        string body_s = string();

        body = getElementContent(ptrData, TEXT("Body"), NULL); 
    
        if (body == NULL) wsprintf(body, TEXT("dummyBody"));
        
        body_s = string(body);                        
        decodeSpecialChar(body_s);    
        writeTextToFile(localTemp, (wchar_t*)body_s);
        // writeTextToFile(localTemp, body);
    
        if (body != NULL)
            delete body;

        if (dbg) 
            MessageBox (NULL, TEXT("written"), TEXT ("Note"), MB_SETFOREGROUND |MB_OK);
        
    }
    else if (dataType == constFavorites) {
        wchar_t internalPath [DIM_LOCAL_TEMP];

        SHGetSpecialFolderPath(NULL, localTemp, CSIDL_FAVORITES, FALSE);
        wsprintf(internalPath, TEXT("%s%s"), localTemp, TEXT("\\"));        
        
        wsprintf(localTemp, TEXT("%s%s"), internalPath, tmp);
        if (dbg) 
            MessageBox (NULL, localTemp, TEXT ("path and file Favorites"), MB_SETFOREGROUND |MB_OK);
        
        if (FILE_SYSTEM_SYNC_SOURCE == TRUE)
            convertAtSlash(localTemp);

        result = GetFileAttributes(localTemp);
        
        if (result == 0xFFFFFFFF || result == -1) {
            if (dbg) {
                wsprintf(tmp, TEXT("%i"), result);
                MessageBox (NULL, tmp, TEXT ("Result no update but new"), MB_SETFOREGROUND |MB_OK);
            }
            if (ptrData != NULL) {
                delete ptrData;
                ptrData = NULL;
            }
            return -1;
        }

        // translate base64 into text
        if (ptrData != NULL) {      
            translateBase642Text(&ptrData);
            // ptrData = translateBase642Text_2(ptrData);
        }

        writeTextToTextFile(localTemp, ptrData);
        if (dbg) 
            MessageBox (NULL, TEXT("written"), TEXT ("Favorites"), MB_SETFOREGROUND |MB_OK);

    }

    if (ptrData != NULL)
        delete ptrData;

    return 0;

}


wchar_t* manageNewItems(SyncItem* item, int dataType, wchar_t* path, wchar_t* dir) {

    LOG.info(TEXT("in manageNewItems ... "));                       
    
    wchar_t localTemp [DIM_LOCAL_TEMP];    
    wchar_t tmp [DIM_LOCAL_TEMP];    
    wchar_t * ptrData   = NULL;
    char* cptrData      = NULL;
    char* content       = NULL;
    byte* reversed      = NULL;
    int num             = 0;
    
    wsprintf(tmp, TEXT("%s"), (wchar_t*)item->getKey(NULL));

    ptrData = (wchar_t*) item->getData();        

    // used to remove start CarriageReturn into return data stream.
    // to be remove when correct into api methods

    removeStartCarriage(&ptrData);            

    // dir from server arrives with \ at the end. It must be removed

    if ( tmp[wcslen(tmp) - 1] == L'\\')
        tmp[wcslen(tmp) - 1] = 0;

    if (dbg) 
        MessageBox (NULL, tmp, TEXT ("tmp"), MB_SETFOREGROUND |MB_OK);
 

    // to create a binaryfile   
    if (dataType == constFiles) {
        if (ptrData != NULL && wcslen(ptrData) > 0) {

            content = wchar2char(ptrData);    
            reversed = new byte[strlen(content) + 1]; 
            memset (reversed, 0, strlen(content) + 1);
            decodeBase64(content, reversed, &num);
        
            if (dbg) {
                wchar_t t[10]; wsprintf(t, TEXT("%i"), num);                        
                MessageBox (NULL, t, TEXT ("num di byte"), MB_SETFOREGROUND |MB_OK);                
            }
               
        }
        wsprintf(localTemp, TEXT("%s%s%s"), dir, TEXT("\\"), tmp);
    
        if (dbg) 
            MessageBox (NULL, localTemp, TEXT ("path and file File"), MB_SETFOREGROUND |MB_OK);

        writeByteToFile(localTemp, reversed, num);
    
        if (content != NULL)
            delete content;
        if (reversed != NULL)
            delete reversed;
    
        wsprintf(tmp, TEXT("%s"), (wchar_t*)item->getKey(NULL));

    } else if (dataType == constNotes) {
        
        // this write the key as the file name
        // wsprintf(localTemp, TEXT("%s%s%s"), dir, TEXT("\\"), tmp);
        
        if (dbg) 
            MessageBox (NULL, localTemp, TEXT ("path and file Note"), MB_SETFOREGROUND |MB_OK);
        
        // translate base64 into text
        if (ptrData != NULL) {      
            translateBase642Text(&ptrData);
            // ptrData = translateBase642Text_2(ptrData);
        }
        
        // parsing of xml structure to retrieve subject as filename and body as text.
        wchar_t* subj = NULL;
        wchar_t* body = NULL;
        string subj_s = string();
        string body_s = string();
        subj = getElementContent(ptrData, TEXT("Subject"), NULL);
        body = getElementContent(ptrData, TEXT("Body"), NULL); 
        
        if (subj == NULL) wsprintf(subj, TEXT("dummySubject"));
        if (body == NULL) wsprintf(body, TEXT("dummyBody"));
        
        subj_s = string(subj);
        body_s = string(body);
        
        decodeSpecialChar(subj_s);    
        decodeSpecialChar(body_s);    
        
        if (length(subj_s) > 20) {
            del(subj_s, 20, length(subj_s) - 20 + 1);
        }        
        wsprintf(localTemp, TEXT("%s%s%s"), dir, TEXT("\\"), (wchar_t*)subj_s);        
        
        if (pos(subj_s, PWI_EXTENSION) == -1) {
            wcscat(localTemp, PWI_EXTENSION);
            wcscat(tmp, PWI_EXTENSION);            
        }
        
        getUniqueFileName(localTemp);
        
        // tmp contains the file name
        getFileName(localTemp, tmp);       

        writeTextToFile(localTemp, (wchar_t*)body_s);

        /*
        if (wcslen(subj) > 20)
            subj[20] = 0;
        
        wsprintf(localTemp, TEXT("%s%s%s"), dir, TEXT("\\"), subj);        
        
        if (wcsstr(subj, PWI_EXTENSION) == NULL) {
            wcscat(localTemp, PWI_EXTENSION);
            wcscat(tmp, PWI_EXTENSION);            
        }
        
        getUniqueFileName(localTemp);
        
        // tmp contains the file name
        getFileName(localTemp, tmp);       

        writeTextToFile(localTemp, body);
        */
        if (dbg) 
            MessageBox (NULL, TEXT("written"), TEXT ("Note"), MB_SETFOREGROUND |MB_OK);
        
        if (subj != NULL)
            delete subj;
        if (body != NULL)
            delete body;

    }
    else if (dataType == constFavorites) {
        wchar_t internalPath [DIM_LOCAL_TEMP];

        SHGetSpecialFolderPath(NULL, localTemp, CSIDL_FAVORITES, FALSE);
        wsprintf(internalPath, TEXT("%s%s"), localTemp, TEXT("\\"));   

        wsprintf(localTemp, TEXT("%s%s"), internalPath,  tmp);
        if (dbg) 
            MessageBox (NULL, localTemp, TEXT ("path and file Favorite"), MB_SETFOREGROUND |MB_OK);
        if (dbg) 
            MessageBox (NULL, ptrData, TEXT ("toWrite"), MB_SETFOREGROUND |MB_OK);
        
        if (FILE_SYSTEM_SYNC_SOURCE == TRUE)
            convertAtSlash(localTemp);
        
        if (dbg) 
            MessageBox (NULL, localTemp, TEXT ("path and file Favorite"), MB_SETFOREGROUND |MB_OK);
        
         // translate base64 into text
        if (ptrData != NULL) {      
            translateBase642Text(&ptrData);
            // ptrData = translateBase642Text_2(ptrData);
        }

        writeTextToTextFile(localTemp, ptrData);       
        if (dbg) 
            MessageBox (NULL, TEXT("writt"), TEXT ("toWrite"), MB_SETFOREGROUND |MB_OK);

        wsprintf(tmp, TEXT("%s"), (wchar_t*)item->getKey(NULL));

    }
        
    //if (ptrData != NULL)
    //    delete ptrData;
    encodeKey(tmp);
    wchar_t* ret = new wchar_t[wcslen(tmp)+ 1];
    wsprintf(ret, tmp);
    return ret;
        

}


void manageDeletedItems(wchar_t* keyExt, int dataType, wchar_t* path, wchar_t* dir) {
   
    LOG.info(TEXT("in managerDeletedItems... "));                   
    wchar_t localTemp [DIM_LOCAL_TEMP];
    
    wchar_t key [DIM_LOCAL_TEMP];
    
    wsprintf(key, keyExt);
    decodeKey (key);
    
    if (dataType == constFiles || dataType == constNotes) {
        wsprintf(localTemp, TEXT("%s%s%s"), dir, TEXT("\\"), key);
        if( DeleteFile(localTemp) == FALSE )
            if (dbg)
                MessageBox (NULL, TEXT("Errore in delete file"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
        else
            if (dbg)
                MessageBox (NULL, TEXT("File deleted"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
    } 
    else if (dataType == constFavorites) {                

        SHGetSpecialFolderPath(NULL, localTemp, CSIDL_FAVORITES, FALSE);
        wcscat(localTemp, TEXT("\\"));        
        wcscat(localTemp, key);        
        
        if (FILE_SYSTEM_SYNC_SOURCE == TRUE)
            convertAtSlash(localTemp);
        // dir from server arrives with \ at the end. It must be removed        

        if ( localTemp[wcslen(localTemp) - 1] == L'\\')
            localTemp[wcslen(localTemp) - 1] = 0;

        if (dbg)
                MessageBox (NULL, localTemp, TEXT ("After \\ removed"), MB_SETFOREGROUND |MB_OK);

        // in favorites, try to delete file. if fails, it's a directory.
        // so enter in dir and delete every files.
        if( DeleteFile(localTemp) == FALSE ) {
            if (dbg)
                MessageBox (NULL, TEXT("Errore in delete file"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);

            if (wcsstr(localTemp, URL_EXTENSION) == NULL) {               
                if (dbg)
                    MessageBox (NULL, TEXT("Try delete dir"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
            
                wchar_t** ptrFilenameInDir      = NULL;
                int countFileInDir              = 0;            
            
                wcscat(localTemp, TEXT("\\"));
                // to retrieve num of files in directory
                getFileList(localTemp, ptrFilenameInDir, &countFileInDir, TRUE, FALSE, NULL);    
                ptrFilenameInDir = new wchar_t*[countFileInDir];
                    
                // to retrieve fileName
                getFileList(localTemp, ptrFilenameInDir, &countFileInDir, FALSE, FALSE, NULL);
            
                for (int i = 0; i < countFileInDir; i++) {
                
                    DeleteFile(ptrFilenameInDir[i]);
                }
                // remove \ at the end of buffer
                localTemp[wcslen(localTemp) - 1] = 0;
                if (dbg)
                    MessageBox (NULL, localTemp, TEXT ("directory"), MB_SETFOREGROUND |MB_OK);
                
                if (RemoveDirectory(localTemp)) {
                    if (dbg)
                        MessageBox (NULL, TEXT("Removed dir"), TEXT ("directory"), MB_SETFOREGROUND |MB_OK);
                } else {
                    if (dbg)
                        MessageBox (NULL, TEXT("NOT Removed dir"), TEXT ("directory"), MB_SETFOREGROUND |MB_OK);
                }
            
                for (i = 0; i < countFileInDir; i++) {
                    if (ptrFilenameInDir[i] != NULL)
                        delete ptrFilenameInDir[i]; 
                }
                if (ptrFilenameInDir != NULL)
                    delete [] ptrFilenameInDir;
            }
            else {}
        }

    }   

}