/*
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//
// @author Marco Magistrali
//

#include <windows.h>
#include <oleauto.h>
#include "initguid.h"
#include "pimstore.h"
#include <stdio.h>
//#include <time.h>

#include "common/Log.h"
#include "spds/common/Utils.h"
#include "spds/common/Constants.h"
#include "spds/common/SyncManagerFactory.h"

#include "spdm/common/DeviceManager.h"
#include "spdm/common/DeviceManagerFactory.h"

#include "pim/LongObj.h"
#include "pim/ActionLog.h"
#include "pim/maincpp.h"
#include "pim/Utils.h"
#include "pim/ContactBuilder.h"
#include "pim/AppointmentBuilder.h"
#include "pim/TaskBuilder.h"
#include "pim/POOMClient2Server.h"
#include "pim/POOMServer2Client.h"
#include "pim/POOMFileManagement.h"
#include "pim/FILEClient2Server.h"
#include "pim/FILEServer2Client.h"
#include "pim/SettingFunctions.h"

static  wchar_t CONTACTS    [15];
static  wchar_t APPOINTMENTS[15];       
static  wchar_t TASKS       [15];
static  wchar_t FAVORITES   [15];
static  wchar_t FILES       [15];
static  wchar_t NOTES       [15];

/*******************************/

/*
* Set all item for SLOW_SYNC. 
* It call the appropriate method to find POOM or FILE items
*
* @param source     : the source to complete with the items
* @param polApp     : the IPOutlookApp object. Used for POOM item (contact, calendar, task)
* @param dataType   : the constant value that represents the contact, calendar... data type
* @param path       : the install path of the SyncClient PIM application on the device
* @param dir        : the directory chose by user to sync briefcase files and notes
*/

void setAllItems(SyncSource &source, IPOutlookApp *polApp, int dataType, wchar_t* path, wchar_t* dir) {
    if (dataType == constFiles || 
        dataType == constNotes || 
        dataType == constFavorites)
        setAllItemsFILE(source, dataType, path, dir);        
    else
        setAllItemsPOOM(source, polApp, dataType);
        
}

/*
* Set modified item for SLOW_SYNC. 
* It call the appropriate method to find new/update/delete POOM or FILE items
*
* @param source     : the source to complete with the items
* @param polApp     : the IPOutlookApp object. Used for POOM item (contact, calendar, task)
* @param dataType   : the constant value that represents the contact, calendar... data type
* @param path       : the install path of the SyncClient PIM application on the device
* @param dir        : the directory chose by user to sync briefcase files and notes
*/

void setModifiedItems(SyncSource &source, IPOutlookApp *polApp, int dataType, wchar_t* path, wchar_t* dir) {
        if (dataType == constFiles || 
            dataType == constNotes || 
            dataType == constFavorites)
        setModifiedItemsFILE(source, polApp, dataType, path, dir);        
     else
        setModifiedItemsPOOM(source, polApp, dataType, path);
}

/*
* Set modified item for SLOW_SYNC. 
* It call the appropriate method to find new/update/delete POOM or FILE items
*
* @param s          : the source to complete with the items
* @param polApp     : the IPOutlookApp object. Used for POOM item (contact, calendar, task)
* @param dataType   : the constant value that represents the contact, calendar... data type
* @param path       : the install path of the SyncClient PIM application on the device
* @param dir        : the directory chose by user to sync briefcase files and notes
*/

void manageReturnedItems(SyncSource& s, IPOutlookApp *polApp, int dataType, wchar_t* path, wchar_t* dir) {
    
    if (dataType == constFiles || 
        dataType == constNotes || 
        dataType == constFavorites)
        manageReturnedItemsFILE(s, dataType, path, dir);
    else
        manageReturnedItemsPOOM(s, polApp, dataType, path);
        

}

/*
* Used for display the mapping when a new item is added. It's for debug only
*
* @param s          : the source to complete with the items
*/

void displayMappings(SyncSource& s) {
    unsigned int n;
    SyncMap** map;

    n = s.getMapSize();
    map = s.getLUIDGUIDMapping();

    LOG.info(TEXT("Mappings"));
    LOG.info(TEXT("========"));
    for (unsigned int i=0; i<n; ++i) {
        wsprintf(logmsg, TEXT("luid: %s, guid: %s"), map[i]->getLUID(NULL), map[i]->getGUID(NULL));
        LOG.info(logmsg);
    }
}

/*
* Set modified item for SLOW_SYNC. 
* It call the appropriate method to find new/update/delete POOM or FILE items
*
* @param syncManager    : the sync manager that drives the sync process
* @param polApp         : the IPOutlookApp object. Used for POOM item (contact, calendar, task)
* @param sourceName     : the the name of the source to sync as ./contact, ./calendar...
* @param path           : the install path of the SyncClient PIM application on the device
* @param errorMessage   : the variable to store the message to return to vb program
* @param sync           : the sync mode as none, two-way
*/

int synchronize(SyncManager* syncManager, IPOutlookApp *polApp, wchar_t* sourceName, 
                wchar_t* path, wchar_t* errorMessage, wchar_t* dir, wchar_t* sync) {
    
    unsigned int ret = 0;

    // this int represents  the type folder object of pocketpcApplication
    int dataType = 0;           

    if (wcscmp(CONTACTS, sourceName) == 0) {        
        ActionLOG.action(SEPARATOR);
        ActionLOG.action(PREPARING_CONTACTS_SYNC);
        ActionLOG.action(SEPARATOR_BLANK);
        dataType = olFolderContacts ;
    }
    else if (wcscmp(APPOINTMENTS, sourceName) == 0){
        ActionLOG.action(SEPARATOR);
        ActionLOG.action(PREPARING_APPOINTMENTS_SYNC);
        ActionLOG.action(SEPARATOR_BLANK);
        dataType = olFolderCalendar;
    }    
    else if (wcscmp(TASKS, sourceName) == 0){
        ActionLOG.action(SEPARATOR);
        ActionLOG.action(PREPARING_TASKS_SYNC);
        ActionLOG.action(SEPARATOR_BLANK);
        dataType = olFolderTasks;
    }   
    else if (wcscmp(FAVORITES, sourceName) == 0){
        ActionLOG.action(SEPARATOR);
        ActionLOG.action(PREPARING_FAVORITES_SYNC);
        ActionLOG.action(SEPARATOR_BLANK);
        dataType = constFavorites;
    }  
    else if (wcscmp(FILES, sourceName) == 0){
        ActionLOG.action(SEPARATOR);
        ActionLOG.action(PREPARING_FILES_SYNC);
        ActionLOG.action(SEPARATOR_BLANK);
        dataType = constFiles;
    }  
    else if (wcscmp(NOTES, sourceName) == 0){
        ActionLOG.action(SEPARATOR);
        ActionLOG.action(PREPARING_NOTES_SYNC);
        ActionLOG.action(SEPARATOR_BLANK);
        dataType = constNotes;
    }  

    SyncSource source = SyncSource(sourceName);    
    
    // "none" is the literal value of SYNC_NONE code    

    if (wcscmp(sync, TEXT("none")) == 0) {
        ActionLOG.action(NO_SYNC);
        goto finally;
    }

    if (syncManager == NULL) {          
        error();
        error(errorMessage);
        return -1;        
    }

    //
    // Initializations
    //    
    
    LOG.info(TEXT("Preparing sync..."));
        
    // for decide what kind of sync to do...
    if (existsFile(dataType, path)) {
        // if file exists is TWO_WAY sync (default)                    
    }
    else {
        source.setPreferredSyncMode(SYNC_SLOW);
        
    }

    ret = syncManager->prepareSync(source);

    if (ret != 0) {
        switch (ret) {
            case ERR_PROTOCOL_ERROR:
                LOG.error(MSG_ERR_PROTOCOL_ERROR );
                wsprintf(errorMessage,MSG_ERR_PROTOCOL_ERROR );
                break;

            case ERR_AUTH_NOT_AUTHORIZED:
                LOG.error(MSG_ERR_AUTH_REQUIRED);
                wsprintf(errorMessage, MSG_ERR_AUTH_REQUIRED);
                break;
            
            case ERR_AUTH_REQUIRED:
                LOG.error(MSG_ERR_AUTH_REQUIRED);
                wsprintf(errorMessage, MSG_ERR_AUTH_REQUIRED);
                break;

            case ERR_AUTH_EXPIRED:
                LOG.error(MSG_ERR_AUTH_EXPIRED);
                wsprintf(errorMessage, MSG_ERR_AUTH_EXPIRED);
                break;

            case ERR_SRV_FAULT:
                LOG.error(MSG_ERR_SRV_FAULT);
                wsprintf(errorMessage, MSG_ERR_SRV_FAULT);
                break;

            case ERR_NOT_FOUND:
                wsprintf(logmsg, MSG_ERR_NOT_FOUND, source.getName(NULL, 0));
                LOG.error(logmsg);
                wsprintf(errorMessage, MSG_ERR_NOT_FOUND, source.getName(NULL, 0));
                break;
            case ERR_INTERNET_CONNECTION_MISSING:
                LOG.error(MSG_ERR_INTERNET_CONNECTION_MISSING);
                wsprintf(errorMessage, MSG_ERR_INTERNET_CONNECTION_MISSING);
                break;               

            default:
                error();
                error(errorMessage);
                break;
        }

        return ret;
    }
                
    switch (source.getSyncMode()) {
                
        case SYNC_SLOW:

            LOG.info(TEXT("Starting SLOW sync..."));
            ActionLOG.action(STARTING_SLOW_SYNC);                       
            setAllItems(source, polApp, dataType, path, dir);           
            break;

        case SYNC_TWO_WAY:                                      
                        
            LOG.info(TEXT("Starting TWO_WAY sync..."));
            ActionLOG.action(STARTING_STD_SYNC);
            setModifiedItems(source, polApp, dataType, path, dir);
            break;
                
        case SYNC_NONE:
            break;

        default:
            break;
    }

    if (source.getSyncMode() == SYNC_NONE) {
        ActionLOG.action(NO_SYNC);
        goto finally;
    }

    LOG.info(TEXT("Starting sync..."));
    ret = syncManager->sync(source);
    
    if (ret != 0) {
        error();
        error(errorMessage);
        if (ret == ERR_CONNECT)
            return ret;
        else
            return -1;
    }
    
    manageReturnedItems(source, polApp, dataType, path, dir);
    
    displayMappings(source);
    
    LOG.info(TEXT("Ending sync..."));
    ActionLOG.action(ENDING_SYNC);
    
    if (syncManager->endSync(source) != 0) {
        
        error();
        error(errorMessage);
        return -1;
    }
                
    
finally:

    SyncItem **p;
    p = source.getNewSyncItems();
    safeDelete((void**)p, source.getNewSyncItemsCount());
    if (p != NULL) {
        delete [] p;
    }
    p = source.getUpdatedSyncItems();
    safeDelete((void**)p, source.getUpdatedSyncItemsCount());
    if (p != NULL) {
        delete [] p;
    }
    p = source.getDeletedSyncItems();
    safeDelete((void**)p, source.getDeletedSyncItemsCount());
    if (p != NULL) {
        delete [] p;
    }
    p = source.getAllSyncItems();
    safeDelete((void**)p, source.getAllSyncItemsCount());
    if (p != NULL) {
        delete [] p;
    }
    
    //
    // Release mapping
    //
    SyncMap** sm;
    sm = source.getLUIDGUIDMapping();
    safeDelete((void**)sm, source.getMapSize());
        
    lastErrorCode = 0;

    return ret;
}

void openLog(wchar_t* path) {
    
    wchar_t value[256];
    value [0] = 0;
    
    getClientConfigurationInternal (NULL, PROPERTY_SOURCEPROP_SPDM_LOG_LEVEL, value, FALSE);
    LOG.setLogPath(path); 
    if (wcscmp(value, TEXT("0")) == 0) {
        Log::debugEnabled = FALSE;    
        Log::infoEnabled = FALSE;    
    } else if (wcscmp(value, TEXT("2")) == 0) {
        Log::debugEnabled = TRUE;    
        Log::infoEnabled = TRUE;    
    } else { // default info
        Log::debugEnabled = FALSE;    
        Log::infoEnabled = TRUE;    
    }   
  
}


/*      
* Entry point from VB application. 
* Accept and manage Variant array of String, long and int. 
* The entry array is a String array in which elements rapresent SyncSource name.
* Now accepted and managed element names are "contacts" and "appointments"
* Array contains at least one element
* @param vbErrorMsg   : the variable to store the message to return to vb GUI interface
* @param path       : the install path of the SyncClient PIM application on the device
*/

DWORD WINAPI synchronization (wchar_t* vbErrorMsg, wchar_t* path) {
        
    /*
    ActionLog::debugEnabled = TRUE;
    ActionLog::setActionPath(path);
    ActionLog::open(true);
    */
    
    openLog(path);
    wchar_t errorMessage[1024];
    int rc          = 0;
    lastErrorCode   = 0;
                
    errorMessage[0] = 0;

    wchar_t source    [60];
    wchar_t dir      [256];      
    wchar_t sync      [60];
    dir[0]  = 0;
    sync[0] = 0;
    /*
    wsprintf(CONTACTS,      TEXT("./contact"   ));
    wsprintf(APPOINTMENTS,  TEXT("./calendar"  ));
    wsprintf(TASKS,         TEXT("./task"      ));
    wsprintf(FAVORITES,     TEXT("./favorite"  ));
    wsprintf(FILES,         TEXT("./briefcase"));
    wsprintf(NOTES,         TEXT("./note"      ));
    */
    wsprintf(CONTACTS,      CONTACTS_NAME    );
    wsprintf(APPOINTMENTS,  APPOINTMENTS_NAME);
    wsprintf(TASKS,         TASKS_NAME       );
    wsprintf(FAVORITES,     FAVORITES_NAME   );
    wsprintf(FILES,         FILES_NAME       );
    wsprintf(NOTES,         NOTES_NAME       );
    
    // Initialization and creation SyncManager
    SyncManagerFactory factory = SyncManagerFactory();
    SyncManager* syncManager = factory.getSyncManager(APPLICATION_URI);
        
    // Initialization and creation IPOutlookApp
    IPOutlookApp    *polApp; // outlook app
    
    HRESULT hr;     


    // for remember code only...
    // int constFolderCalendar  =  9; 
    // int constFolderContacts  = 10;
    // int constFolderTasks     = 13;

    //
    // custom values
    //
    // int constFiles       = 600;
    // int constNotes       = 601;
    // int constFavorites   = 602;
    

    // Initialize COM for Pocket Outlook
    if (FAILED (CoInitializeEx (NULL, 0))) return -2;

    // Get the application object
    hr = CoCreateInstance (CLSID_Application, NULL, CLSCTX_INPROC_SERVER, IID_IPOutlookApp, (LPVOID*)&polApp);
    if (FAILED(hr)) return -2;

    // Log on to Pocket Outlook
    hr = polApp->Logon (NULL);
    if (FAILED (hr)) return -2;
    
    if (FAILED(hr)) return -2;              

    Config* config = NULL;

    config = new Config(APPLICATION_URI);
    BOOL ret = config->readFromDM();

    int sourceCount = config->getSyncSourceCount();

    for (int i = 0; i < sourceCount; i++) {
    config->getSyncSourceConfig(i).getName(source);
    config->getSyncSourceConfig(i).getDir(dir);
    config->getSyncSourceConfig(i).getSync(sync);
    
    if (wcscmp(CONTACTS, source) == 0) {
                        
        if (dbg) {
            MessageBox (NULL, TEXT("In Contacts"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
            MessageBox (NULL, path, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
        }
        rc = synchronize(syncManager, polApp, CONTACTS, path, errorMessage, dir, sync);
                            
        if (dbg) {
            MessageBox (NULL, errorMessage, TEXT ("ErrorMessage"), MB_SETFOREGROUND |MB_OK);
            MessageBox (NULL, TEXT("End contacts sync"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
        }

    }
    else if (wcscmp(APPOINTMENTS, source) == 0){

        if (dbg)
            MessageBox (NULL, TEXT("In Appointments"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
    
        rc = synchronize(syncManager, polApp, APPOINTMENTS, path, errorMessage, dir, sync);

        if (dbg)
            MessageBox (NULL, TEXT("End appointmens sync"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);

    }
    else if (wcscmp(TASKS, source) == 0){

        if (dbg)
            MessageBox (NULL, TEXT("In Tasks"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
    
        rc = synchronize(syncManager, polApp, TASKS, path, errorMessage, dir, sync);

        if (dbg)
            MessageBox (NULL, TEXT("End tasks sync"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);

    }
    else if (wcscmp(FAVORITES, source) == 0){

        if (dbg)
            MessageBox (NULL, TEXT("In Favorites"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
    
        rc = synchronize(syncManager, polApp, FAVORITES, path, errorMessage, dir, sync);

        if (dbg)
            MessageBox (NULL, TEXT("End Favorites sync"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);

    }
    else if (wcscmp(FILES, source) == 0){

        if (dbg)
            MessageBox (NULL, TEXT("In File"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
    
        rc = synchronize(syncManager, polApp, FILES, path, errorMessage, dir, sync);

        if (dbg)
            MessageBox (NULL, TEXT("End File sync"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);

    }
    else if (wcscmp(NOTES, source) == 0){

        if (dbg)
            MessageBox (NULL, TEXT("In Note"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
    
        rc = synchronize(syncManager, polApp, NOTES, path, errorMessage, dir, sync);

        if (dbg)
            MessageBox (NULL, TEXT("End Note sync"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);

    }
    else {
        if (dbg)
            MessageBox (NULL, TEXT("Nothing to sync"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
    }
        
    if (dbg) {
        wchar_t t[10];
        wsprintf(t, TEXT("%i"), rc);
        MessageBox (NULL, t, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);

    }

    if (rc != 0)
        break;
    }

    wsprintf(vbErrorMsg, errorMessage);
    vbErrorMsg[wcslen(vbErrorMsg)] = L' ';
    
    // Log off and release the Application object
    polApp->Logoff ();
    polApp->Release ();


    delete syncManager;
    LOG.info(TEXT("Sync ended."));
    ActionLOG.action(SYNC_ENDED);
   
    return rc;
}



/********************************************************
END sync
********************************************************/




