/*
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//
// @author Marco Magistrali
//

#include <windows.h>


#include "pimstore.h"
#include <stdio.h>

#include "common/Log.h"
#include "spds/common/Utils.h"
#include "spds/common/Constants.h"
#include "spds/common/SyncManagerFactory.h"

#include "pim/LongObj.h"

#include "pim/ActionLog.h"
#include "pim/Utils.h"
#include "pim/maincpp.h"
#include "pim/ContactBuilder.h"
#include "pim/AppointmentBuilder.h"
#include "pim/TaskBuilder.h"
#include "pim/POOMClient2Server.h"
#include "pim/POOMFileManagement.h"


void setAllItemsPOOM(SyncSource &source, IPOutlookApp *polApp, int dataType) {          

    LOG.info(TEXT("SLOW: in setAllItems... "));             
    if (dbg)
        MessageBox (NULL, TEXT("SLOW: in setAllItems... "), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
    // only if dataType is appointment
    if (dataType == olFolderCalendar)
        normalizeAppointment(polApp, dataType);

    wchar_t localTemp [DIM_LOCAL_TEMP];

    IPOutlookItemCollection *pItems;
    IFolder *pFolder;

    IContact     *pContact;
    IAppointment *pAppointment;
    ITask        *pTask;

    polApp->GetDefaultFolder (dataType, &pFolder);

    pFolder->get_Items (&pItems);

    int countItems = 0;
    objlist currentOid  = objlist(true);
    objlist currentHash = objlist(true);

    pItems->get_Count(&countItems);
    if (dbg) {
        wsprintf(localTemp, TEXT("AllItems: Item's number is: %i"), countItems);
        MessageBox (NULL, localTemp, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
    }       
    SyncItem** allItems = new SyncItem* [countItems];               
    for (int j=0; j<countItems; j++) {
        string stringItem = string();
        long oid = 0;

        // Item values start from element number 1 not 0!!!!!
        // from dataType choose if a Contact or a Appointment
        // leave 2 populateString to semplify
        if (dataType == olFolderContacts) {
            pItems->Item (j + 1, (IDispatch**)&pContact);
            populateContactStringItem(stringItem, pContact);
            pContact->get_Oid(&oid);
            
            long currentLongHash = calculateContactHash(pContact);                  
            // set these objlist to write to file...
            add(currentOid, new LongObj(oid));
            add(currentHash, new LongObj(currentLongHash));
                
        }
        else if (dataType == olFolderCalendar){
            pItems->Item (j + 1, (IDispatch**)&pAppointment);
            populateAppointmentStringItem(stringItem, pAppointment);
            pAppointment->get_Oid(&oid);

            long currentLongHash = calculateAppointmentHash(pAppointment);
            
            // set these objlist to write to file...
            add(currentOid, new LongObj(oid));
            add(currentHash, new LongObj(currentLongHash));
                
        }
        else if (dataType == olFolderTasks){
            pItems->Item (j + 1, (IDispatch**)&pTask);
            populateTaskStringItem(stringItem, pTask);
            pTask->get_Oid(&oid);

            long currentLongHash = calculateTaskHash(pTask);
            
            // set these objlist to write to file...
            add(currentOid, new LongObj(oid));
            add(currentHash, new LongObj(currentLongHash));
                
        }
        wsprintf(localTemp, TEXT("%i"), oid);                                   
        // Set the SyncItem
        allItems[j] = new SyncItem(localTemp);                  
        
        //stringItem = CDATA_START + stringItem;
        //stringItem += CDATA_END;

        // encode base64
        
        wchar_t* t = NULL;
        // t = encodeBase64W((wchar_t*)stringItem);
        t = encodeBase64UTF8((wchar_t*)stringItem);
        stringItem = t;

        allItems[j]->setData((wchar_t*)stringItem, ((length(stringItem) + 1)*sizeof(wchar_t))); 

        if (t != NULL)
            delete t;
                
    }

    source.setAllSyncItems(allItems, countItems);

    // Release objects
    if (countItems != 0){
        if (dataType == olFolderContacts)
            pContact->Release ();
        else if (dataType == olFolderCalendar)
            pAppointment->Release ();
        else if (dataType == olFolderTasks)
            pTask->Release ();
    }
    pItems->Release ();
    pFolder->Release ();
    
    wsprintf(localTemp, SET_ALL_ITEMS, countItems); 
    ActionLOG.action(localTemp);
    LOG.info(TEXT("Exiting from setAllItems... "));

 }

 /*
Set modified Items for FAST_SYNC from PPC to server
*/

void setModifiedItemsPOOM(SyncSource &source, IPOutlookApp *polApp, int dataType, wchar_t* path) {
        
    LOG.info(TEXT("FAST: in setModifiedItems... "));                        
    wchar_t localTemp [DIM_LOCAL_TEMP];
    
    if (dbg)
        MessageBox (NULL, TEXT("FAST: in setModifiedItems... "), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);


    IContact     *pContact;
    IAppointment *pAppointment;
    ITask        *pTask;   

    int countItems = 0;
    
    objlist newOid = objlist(true);
    objlist updatedOid = objlist(true);
    objlist deletedOid = objlist(true);
    
    // only if dataType is appointment
    if (dataType == olFolderCalendar)
            normalizeAppointment(polApp, dataType);


    fillModifiedOidArray (newOid, updatedOid, deletedOid, polApp, dataType, path) ;
            
    SyncItem** newItems = new SyncItem* [length(newOid)];
    SyncItem** updatedItems = new SyncItem* [length(updatedOid)];
    SyncItem** deletedItems = new SyncItem* [length(deletedOid)];

    // for every oid array...
    int i = 0;
    long oid = 0;
    for (i = 0; i < length(newOid); i++) {
            
        string stringItem = string();
        LongObj* longObj = (LongObj*) get(newOid, i);
                
        oid = longObj->getOID();                
        wsprintf(localTemp, TEXT("%i"), oid);
        

        if (dataType == olFolderContacts) { // it's a contact
            polApp->GetItemFromOid (oid, (IDispatch**)&pContact);
            populateContactStringItem(stringItem, pContact);        
        }
        else if (dataType == olFolderCalendar){ // it's an appointments
        
            polApp->GetItemFromOid (oid, (IDispatch**)&pAppointment);
            populateAppointmentStringItem(stringItem, pAppointment);
                                
        }
        else if (dataType == olFolderTasks){ // it's an task
            
            polApp->GetItemFromOid (oid, (IDispatch**)&pTask);
            populateTaskStringItem(stringItem, pTask);
                                    
        }
        // Set the SyncItem
            
        newItems[i] = new SyncItem(localTemp);                          
                
        //stringItem = CDATA_START + stringItem;
        //stringItem += CDATA_END;                       

        // encode base64
        wchar_t* t = NULL;        
        // t = encodeBase64W((wchar_t*)stringItem);
        t = encodeBase64UTF8((wchar_t*)stringItem);
        stringItem = t;
        
        newItems[i]->setData((wchar_t*)stringItem, ((length(stringItem) + 1)*sizeof(wchar_t))); 
                
        if (t != NULL)
            delete t;
        
    }

        
    i=0;
    oid = 0;
    for (i = 0; i < length(updatedOid); i++) {
                
        string stringItem = string();
        LongObj* longObj = (LongObj*) get(updatedOid, i);
                
        oid = longObj->getOID();                
        wsprintf(localTemp, TEXT("%i"), oid);
            

        if (dataType == olFolderContacts) { // it's a contact
                polApp->GetItemFromOid (oid, (IDispatch**)&pContact);
                populateContactStringItem(stringItem, pContact);                        
        }
        else if (dataType == olFolderCalendar){ // it's an appointments
                polApp->GetItemFromOid (oid, (IDispatch**)&pAppointment);
                populateAppointmentStringItem(stringItem, pAppointment);                                        
        }
        else if (dataType == olFolderTasks){ // it's a task
                polApp->GetItemFromOid (oid, (IDispatch**)&pTask);
                populateTaskStringItem(stringItem, pTask);                                      
        }
        // Set the SyncItem
        updatedItems[i] = new SyncItem(localTemp);              
            
        //stringItem = CDATA_START + stringItem;
        //stringItem += CDATA_END;

        // encode base64
        wchar_t* t = NULL;
        // t = encodeBase64W((wchar_t*)stringItem);
        t = encodeBase64UTF8((wchar_t*)stringItem);
        stringItem = t;

        updatedItems[i]->setData((wchar_t*)stringItem, ((length(stringItem) + 1)*sizeof(wchar_t)));                                                                             
            
        if (t != NULL)
            delete t;
    }
        
    i=0;
    oid = 0;
    for (i = 0; i < length(deletedOid); i++) {
            
        string stringItem = string();
        LongObj* longObj = (LongObj*) get(deletedOid, i);
                
        oid = longObj->getOID();                
        wsprintf(localTemp, TEXT("%i"), oid);
            

        if (dataType == olFolderContacts) { // it's a contact           
        }
        else if (dataType == olFolderCalendar){ // it's an appointments                                 
        }
        else if (dataType == olFolderTasks) {                                   
        }
                    // Set the SyncItem
        deletedItems[i] = new SyncItem(localTemp);                                      
                    
    }

    // now we have to disctinct if generic Items are IContact or ICalendar
    if ((length(newOid) != 0)  || (length(updatedOid) != 0) ) {
        if (dataType == olFolderContacts)
                pContact->Release();
        else if (dataType == olFolderCalendar)
                pAppointment->Release ();
        else if (dataType == olFolderTasks)
                pTask->Release ();
    }
        
    if (length(newOid) > 0)
        source.setNewSyncItems(newItems, length(newOid));
    if (length(deletedOid) > 0)
        source.setDeletedSyncItems(deletedItems, length(deletedOid));
    if (length(updatedOid) > 0)
        source.setUpdatedSyncItems(updatedItems, length(updatedOid));
    
    ActionLOG.action(MESSAGE_TO_SERVER);
    wsprintf(localTemp, CLIENT_NEW_ITEMS, length(newOid));
    ActionLOG.action(localTemp);
    wsprintf(localTemp, CLIENT_UPDATE_ITEMS, length(updatedOid));
    ActionLOG.action(localTemp);
    wsprintf(localTemp, CLIENT_DELETE_ITEMS, length(deletedOid));
    ActionLOG.action(localTemp);

    LOG.info(TEXT("end setModifiedItems... "));
}


/*
fill array new, update, delete with OID of element to modify. This function can be used by every source.
This function call funcion that use the alghorytm to retrieve OID modified
*/
void fillModifiedOidArray (objlist& newOid, objlist& updateOid, objlist& deleteOid, 
                                                   IPOutlookApp *polApp, int dataType, wchar_t* path) {
        
        
    wchar_t localTemp [DIM_LOCAL_TEMP];

    IPOutlookItemCollection *pItems;
    IFolder *pFolder;

    IContact     *pContact;
    IAppointment *pAppointment;
    ITask        *pTask;

    polApp->GetDefaultFolder (dataType, &pFolder);

    pFolder->get_Items (&pItems);

    int currentCountItems = 0;
    objlist currentOid  = objlist(true);
    objlist currentHash = objlist(true);

    pItems->Sort(TEXT ("[Oid]"), FALSE);
    pItems->get_Count(&currentCountItems);
    
    if (dbg) {
            wsprintf(localTemp, TEXT("In device: Item's number is: %i"), currentCountItems);
            MessageBox (NULL, localTemp, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
    }
    long previousCountItems = 0;
    objlist previousOid  = objlist(true);
    objlist previousHash = objlist(true);


    readFromFile(&previousCountItems, previousOid, previousHash, dataType, path);

    if (dbg) {
            wsprintf(localTemp, TEXT("%i"), previousCountItems);
            MessageBox (NULL, localTemp, TEXT ("Previoes count item"), MB_SETFOREGROUND |MB_OK);
    }
        
    int i = 0;

    int j = 0;
    int lastFound = -1;

    long currentLongOid = 0;
    long currentLongHash = 0;
    
    for (i = 0; i < currentCountItems; i++) {

        if (dataType == olFolderContacts) {
            currentLongOid = 0;
            pItems->Item (i + 1, (IDispatch**)&pContact);
            
            pContact->get_Oid(&currentLongOid);                     
            currentLongHash = calculateContactHash(pContact);
            // set these objlist to write to file...
            add(currentOid, new LongObj(currentLongOid));
            add(currentHash, new LongObj(currentLongHash));

        }
        else if (dataType == olFolderCalendar){ // for appointment
            long internalOid = 0;
            
            pItems->Item (i + 1, (IDispatch**)&pAppointment);
            
            pAppointment->get_Oid(&internalOid);
            currentLongHash = calculateAppointmentHash(pAppointment);
            currentLongOid = internalOid;
            // set these objlist to write to file...
            add(currentOid, new LongObj(currentLongOid));
            add(currentHash, new LongObj(currentLongHash));
                
        }
        else if (dataType == olFolderTasks){ // for appointment
            long internalOid = 0;                   
            pItems->Item (i + 1, (IDispatch**)&pTask);                      
            pTask->get_Oid(&internalOid);
            currentLongHash = calculateTaskHash(pTask);
            currentLongOid = internalOid;
            // set these objlist to write to file...
            add(currentOid, new LongObj(currentLongOid));
            add(currentHash, new LongObj(currentLongHash));
                
        }
        int itemFound = 0;
        j = lastFound + 1;
        
        if (dbg) {
            wsprintf(localTemp, TEXT("i: %i - j: %i"), i,j);
            MessageBox (NULL, localTemp, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
        }
        while ((j < previousCountItems && itemFound == 0)) {
            LongObj* longObjOid = (LongObj*) get(previousOid, j);
            long previousLongOid = longObjOid->getOID();    


            if (currentLongOid == previousLongOid) {
                itemFound = 1;
                lastFound = j;
                
                LongObj* longObjHash = (LongObj*) get(previousHash, j);
                long previousLongHash = longObjHash->getOID();  

                if (currentLongHash != previousLongHash) {
                    // if different hash then into updateOid to pass to setModifiedItems
                    if (dbg)
                            MessageBox (NULL, TEXT("Update"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
                    add(updateOid, new LongObj(currentLongOid));
                } else {
                    // nothing to do...
                }
            }
            j++;
        }
        if (itemFound == 0) {
            if (dbg)
                MessageBox (NULL, TEXT("New"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
            add(newOid, new LongObj(currentLongOid));
        }
                

    } // fine for

    i = 0;
    lastFound = -1;

    for (j = 0; j < previousCountItems; j++) {

        int itemFound = 0;
        i = lastFound + 1;
        
        LongObj* longObjOid = (LongObj*) get(previousOid, j);
        long previousLongOid = longObjOid->getOID();    

        
        while ((i < currentCountItems && itemFound == 0)) {
            LongObj* longObjOidC = (LongObj*) get(currentOid, i);
            long currentLongOid = longObjOidC->getOID();    

            if (currentLongOid == previousLongOid) {
                itemFound = 1;
                lastFound = i;
            }
            i++;
        }

        if (itemFound == 0) {
            if (dbg)
                MessageBox (NULL, TEXT("Delete"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
            add(deleteOid, new LongObj(previousLongOid));
        }

    } // fine for
        
    if (dbg)
        MessageBox (NULL, TEXT("Prima di write"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
    

    // Release objects
    if (currentCountItems != 0){
        if (dataType == olFolderContacts)
            pContact->Release ();
        else if (dataType == olFolderCalendar)
            pAppointment->Release ();
        else if (dataType == olFolderTasks)
            pTask->Release ();
    }
    
    pFolder->Release ();
    pItems->Release ();
}