/*
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//
// @author Marco Magistrali
//

#include <windows.h>


#include "pimstore.h"
#include <stdio.h>

#include "common/Log.h"
#include "spds/common/Utils.h"
#include "spds/common/Constants.h"
#include "spds/common/SyncManagerFactory.h"

#include "pim/LongObj.h"

#include "pim/ActionLog.h"
#include "pim/Utils.h"
#include "pim/maincpp.h"
#include "pim/ContactBuilder.h"
#include "pim/AppointmentBuilder.h"
#include "pim/TaskBuilder.h"
#include "pim/POOMServer2Client.h"
#include "pim/POOMFileManagement.h"



void manageReturnedItemsPOOM(SyncSource& s, IPOutlookApp *polApp, int dataType, wchar_t* path) {
    unsigned int n, i, j = 0;
    SyncItem** items;   
    SyncItem** updatedSyncItems;
    wchar_t localTemp [DIM_LOCAL_TEMP];

    ActionLOG.action(MESSAGE_FROM_SERVER);

    //UPDATE************
    
    n = s.getUpdatedSyncItemsCount();
    updatedSyncItems = s.getUpdatedSyncItems();

    long* oidToInsert = NULL;

    if (n > 0) {
            
        oidToInsert = new long[n];      
                        
        for (i=0; i<n; ++i) {
            oidToInsert[i] = NULL; // every oidToInsert pointer initialized to null!
        }
    }
                
    wsprintf(localTemp,SERVER_UPDATE_ITEMS, n );
    ActionLOG.action(localTemp);

    if (dbg) {
        wsprintf(localTemp, TEXT("update item are: %i"), n );
        MessageBox (NULL, localTemp, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
    }

    LOG.info(TEXT("Updated items"));
    LOG.info(TEXT("============="));
    for (i = 0; i < n; ++i) {                                       

        if (manageUpdatedItems(updatedSyncItems[i], polApp, dataType) != S_OK) {

            if (dbg) {
                wsprintf(localTemp, TEXT("i = : %i"), i );
                MessageBox (NULL, localTemp, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
                MessageBox (NULL, (wchar_t*)updatedSyncItems[i]->getData(), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
            }
            oidToInsert[j] = i;
            j++;
        }
    }
    
   

    //NEW *********************

    n = s.getNewSyncItemsCount();
    items = s.getNewSyncItems();
        
    wsprintf(localTemp,SERVER_NEW_ITEMS, n );
    ActionLOG.action(localTemp);

    // array in which store the new oid for the mapping...
    // delete in the end every pointer...
    if (dbg) {      
        wsprintf(localTemp, TEXT("Return new item are: %i"), n );
        MessageBox (NULL, localTemp, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
    }
    
    long* newOidArray = NULL;
    
    wchar_t luid [DIM_KEY];
    SyncMap**  mappings = NULL;
    
    // da fare solo se n > 0!!!
    if (n + j> 0) {
            
        newOidArray = new long[n + j];          
        
        mappings = new SyncMap* [n + j];
        for (i=0; i<n + j; ++i) {
            mappings[i] = NULL; // every map pointer initialized to null!
        }
    }
        
    LOG.info(TEXT("New items"));
    LOG.info(TEXT("========="));
    for (i=0; i<n; ++i) {
                
        newOidArray[i] = manageNewItems(items[i], polApp, dataType);
        wsprintf(luid, TEXT("%i"), newOidArray[i] );
        
        if (dbg) {
            MessageBox (NULL, luid, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
                    
            wsprintf(localTemp, TEXT("chiave: %s"), items[i]->getKey(NULL) );
            MessageBox (NULL, localTemp, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);

            wsprintf(localTemp, TEXT("luid: %s"), luid );
            MessageBox (NULL, localTemp, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
        }

        mappings[i] = new SyncMap(items[i]->getKey(NULL), luid);
    }
   int k = 0;  
   for (i = n; i < n + j; ++i) {

        newOidArray[i] = manageNewItems(updatedSyncItems[oidToInsert[k]], polApp, dataType);
        wsprintf(luid, TEXT("%i"), newOidArray[i] );
        
        if (dbg) {
            MessageBox (NULL, luid, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
                    
            wsprintf(localTemp, TEXT("chiave: %s"), updatedSyncItems[oidToInsert[k]]->getKey(NULL) );
            MessageBox (NULL, localTemp, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);

            wsprintf(localTemp, TEXT("luid: %s"), luid );
            MessageBox (NULL, localTemp, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
        }

        mappings[i] = new SyncMap(updatedSyncItems[oidToInsert[k]]->getKey(NULL), luid);
        k ++;
    }
        
        
    // mapping for the new element if and only if i > 0
    if (n + j> 0) {         
        s.setLUIDGUIDMapping(mappings, n + j);          
    }

    //DELETE*******************

    n = s.getDeletedSyncItemsCount();
    items = s.getDeletedSyncItems();
        

    wsprintf(localTemp,SERVER_DELETE_ITEMS, n );
    ActionLOG.action(localTemp);

    if (dbg) {
        wsprintf(localTemp, TEXT("delete item are: %i"), n );
        MessageBox (NULL, localTemp, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
    }

    LOG.info(TEXT("Deleted items"));
    LOG.info(TEXT("============="));
    for (i=0; i<n; ++i) {
       
       manageDeletedItems(items[i]->getKey(NULL), polApp, dataType);
    }
        
    // write file with actual oid and hash for contact or appointment
    writeCurrentItems(polApp,dataType, path);
        

}

long manageNewItems(SyncItem* item, IPOutlookApp *polApp, int dataType) {

    LOG.info(TEXT("in manageNewItems ... "));                       
    
    long oidNew = 0;
    //wchar_t localTemp [DIM_LOCAL_TEMP];
    if (dbg)        
        MessageBox (NULL, TEXT("In newItem"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
    
    IFolder *pFolder;
    IPOutlookItemCollection *pItems;

    wchar_t * ptrData = NULL;
    ptrData = (wchar_t*) item->getData();
    if (dbg)        
        MessageBox (NULL, ptrData, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
    // decode b64
    wchar_t* t = NULL;
    // t = decodeBase64W(ptrData);
    t = decodeBase64WUTF8(ptrData);    
    delete ptrData;    
    ptrData = t;
    
     if (dbg)        
        MessageBox (NULL, ptrData, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);

    string ptrDataString = string();
    ptrDataString += ptrData;
    replaceAmpLGt(ptrDataString);
    
     if (dbg)        
        MessageBox (NULL, ptrData, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);

    ptrData = (wchar_t*)ptrDataString;

    // to create contacts   
    if (dataType == olFolderContacts) {
        IContact *pContact;
        
        // Create a new contact.
        polApp->GetDefaultFolder (olFolderContacts, &pFolder);
        pFolder->get_Items (&pItems);
        pItems->Add((IDispatch**)&pContact);
        
        // function to complete contact and fill it
        completeContact(pContact, ptrData);

        // Save the new contact.
        pContact->Save ();
        pContact->get_Oid(&oidNew);

        // Release objects
        pContact->Release ();
        

    } else if (dataType == olFolderCalendar) {
        IAppointment *pAppointment;

        polApp->GetDefaultFolder (olFolderCalendar, &pFolder);
        pFolder->get_Items (&pItems);
        pItems->Add((IDispatch**)&pAppointment);
        
        completeAppointment(pAppointment, ptrData);

        // Save the new appointment.
        pAppointment->Save ();
        pAppointment->get_Oid(&oidNew);

        // Release objects
        pAppointment->Release ();
        
    } else if (dataType == olFolderTasks) {
        ITask *pTask;

        polApp->GetDefaultFolder (olFolderTasks, &pFolder);
        pFolder->get_Items (&pItems);
        pItems->Add((IDispatch**)&pTask);
        
        completeTask(pTask, ptrData);

        // Save the new appointment.
        pTask->Save ();
        pTask->get_Oid(&oidNew);

        // Release objects
        pTask->Release ();

    }
    
    //wsprintf(localTemp, TEXT("%i"), oidNew);
    //MessageBox (NULL, localTemp, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
    
    pFolder->Release ();
    pItems->Release ();
            
    return oidNew;
    

}

/*
from server to PPC
*/

long manageUpdatedItems(SyncItem* item, IPOutlookApp *polApp, int dataType) {


    LOG.info(TEXT("in managerUpdatedItems ... "));                  


    wchar_t* dummy = NULL;
    wchar_t* dummyEvent = NULL;
    
    long oidToUpdate = 0;
    oidToUpdate = wcstol(item->getKey(NULL), &dummy, 0);
    HRESULT h = NULL;
    
    wchar_t * ptrData = NULL;
    
    int size = 0;
    size = wcslen((wchar_t*) item->getData()) + 1;
    ptrData = new wchar_t [size];
    wsprintf(ptrData, (wchar_t*) item->getData());
    
    // ptrData = (wchar_t*) item->getData();
     
    // decode b64
    wchar_t* t = NULL;
    //t = decodeBase64W(ptrData);
    t = decodeBase64WUTF8(ptrData);    
    if (ptrData != NULL)
        delete ptrData; 
    
    ptrData = t;

    string ptrDataString = string();
    ptrDataString += ptrData;
    replaceAmpLGt(ptrDataString);
    
    ptrData = (wchar_t*)ptrDataString;
    
        // to update contacts
    if (dataType == olFolderContacts) {

        DATE date = NULL;                               

        IContact *pContact;     
        h = polApp->GetItemFromOid (oidToUpdate, (IDispatch**)&pContact);
        if (h == S_OK) {
            // function to complete contact and fill it!
            completeContact(pContact, ptrData);

            pContact->Save();
            pContact->Release();
        } else {
            if (dbg) {
                    MessageBox (NULL, TEXT("insert as new contact item..."), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
            }
        }
    }
    else if (dataType == olFolderCalendar){ 
        IAppointment *pAppointment;

        h = polApp->GetItemFromOid (oidToUpdate, (IDispatch**)&pAppointment);           
        
        if (h == S_OK) {
            completeAppointment(pAppointment, ptrData);

            pAppointment->Save();
            pAppointment->Release();
        } else {
            if (dbg) {
                MessageBox (NULL, TEXT("insert as new appointment item..."), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
            }
        }
                
    } 
    else if (dataType == olFolderTasks){        
        ITask *pTask;

        h = polApp->GetItemFromOid (oidToUpdate, (IDispatch**)&pTask);          
        
        if (h == S_OK) {
            completeTask(pTask, ptrData);

            pTask->Save();
            pTask->Release();
        } else {
            if (dbg) {
                MessageBox (NULL, TEXT("insert as new Task item..."), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
            }
        }
            
    }            
    
    if (ptrData != NULL)
        delete ptrData;

    return h;

}


/*
Use to delete item from server to PPC
*/

void manageDeletedItems(wchar_t* key, IPOutlookApp *polApp, int dataType) {

    LOG.info(TEXT("in managerDeletedItems... "));                   
    
    long oidToDelete = 0;
    wchar_t* local = NULL;
    oidToDelete = wcstol(key, &local, 0);
    HRESULT h = NULL;
    // to delete contacts
    if (dataType == olFolderContacts) {
        IContact *pContact;     
        h = polApp->GetItemFromOid (oidToDelete, (IDispatch**)&pContact);               
        if (h == S_OK) {
            pContact->Delete();
            pContact->Save();
            pContact->Release();
        }
            
    }
    // to delete appointments
    else if (dataType == olFolderCalendar){ 
        IAppointment *pAppointment;
        h = polApp->GetItemFromOid (oidToDelete, (IDispatch**)&pAppointment);
        if (h == S_OK) {
            pAppointment->Cancel();         
            pAppointment->Delete();
            pAppointment->Release();
        }
            
    }
    // to delete tasks
    else if (dataType == olFolderTasks){        
        ITask *pTask;
        h = polApp->GetItemFromOid (oidToDelete, (IDispatch**)&pTask);
        if (h == S_OK) {    
            pTask->Delete();
            pTask->Release();
        }           
    }
        

}


void writeCurrentItems(IPOutlookApp *polApp, int dataType, wchar_t* path) {     
        
    wchar_t localTemp [DIM_LOCAL_TEMP];

    IPOutlookItemCollection *pItems;
    IFolder *pFolder;

    IContact     *pContact;
    IAppointment *pAppointment;
    ITask        *pTask;

    polApp->GetDefaultFolder (dataType, &pFolder);
    
    pFolder->get_Items (&pItems);
    
    pItems->Sort(TEXT ("[Oid]"), FALSE);
    int countItems = 0;
    objlist currentOid  = objlist(true);
    objlist currentHash = objlist(true);

    pItems->get_Count(&countItems);
    
    if (dbg) {
            wsprintf(localTemp, TEXT("writeCurrentItems: Item's number is: %i"), countItems);
            MessageBox (NULL, localTemp, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
    }
    
    for (int j=0; j<countItems; j++) {
        string stringItem = string();
        long oid = 0;

        // Item values start from element number 1 not 0!!!!!
        // from dataType choose if a Contact or a Appointment
        // leave 2 populateString to semplify
        if (dataType == olFolderContacts) {
            pItems->Item (j + 1, (IDispatch**)&pContact);           
            pContact->get_Oid(&oid);
            
            long currentLongHash = calculateContactHash(pContact);                  
            // set these objlist to write to file...
            add(currentOid, new LongObj(oid));
            add(currentHash, new LongObj(currentLongHash));

                
        }
        else if (dataType == olFolderCalendar){
            pItems->Item (j + 1, (IDispatch**)&pAppointment);
            
            pAppointment->get_Oid(&oid);

            long currentLongHash = calculateAppointmentHash(pAppointment);
            
            // set these objlist to write to file...
            add(currentOid, new LongObj(oid));
            add(currentHash, new LongObj(currentLongHash));
                
        }
        else if (dataType == olFolderTasks){
            pItems->Item (j + 1, (IDispatch**)&pTask);
            
            pTask->get_Oid(&oid);

            long currentLongHash = calculateTaskHash(pTask);
            
            // set these objlist to write to file...
            add(currentOid, new LongObj(oid));
            add(currentHash, new LongObj(currentLongHash));
                
        }

    }
        
    writeToFile(currentOid, currentHash, dataType, path);
    
    // Release objects
    if (countItems != 0){
        if (dataType == olFolderContacts)
            pContact->Release ();
        else if (dataType == olFolderCalendar)
            pAppointment->Release ();
        else if (dataType == olFolderTasks)
            pTask->Release ();
    }
    pItems->Release ();
    pFolder->Release ();
        
}
