/*
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//
// @author Marco Magistrali
//

#include "spds/common/Config.h"
#include "spds/common/Constants.h"

#include "spdm/common/DeviceManager.h"
#include "spdm/common/DeviceManagerFactory.h"
#include "pim/SettingFunctions.h"
#include "pim/Utils.h"
#include <winioctl.h>
#include "ptypes/ptypes.h"

#define DIM_MANAGEMENT_PATH 512
#define DIM_DEVICE_ID       50

#define ROOT_CONTEXT        TEXT("Funambol/SyncclientPIM")
#define SPDM_SYNCURL        TEXT("spdm.syncURL")
#define SPDM_SERVERNAME     TEXT("spdm.serverName")
#define SPDM_PASSWORD       TEXT("spdm.password")
#define SPDM_USERNAME       TEXT("spdm.username")
#define SPDM_DEVICEID       TEXT("spdm.deviceId")

#define FAST                TEXT("fast")
#define TWO_WAY             TEXT("two-way")
#define SLOW                TEXT("slow")
#define NONE                TEXT("none");
#define SYNC_CONTACTS       TEXT("sync.contacts")

#define SYNC_APPOINTMENTS   TEXT("sync.appointments")
#define SYNC_TASKS          TEXT("sync.tasks")
#define SYNC_FILES          TEXT("sync.files")
#define SYNC_NOTES          TEXT("sync.notes")
#define SYNC_FAVORITES      TEXT("sync.favorites")

#define HTTP_HEADER         TEXT("http://")





#define IOCTL_HAL_GET_DEVICEID CTL_CODE(FILE_DEVICE_HAL, 21, METHOD_BUFFERED, FILE_ANY_ACCESS)

extern "C" __declspec(dllimport) BOOL KernelIoControl(
                                                        DWORD dwIoControlCode, 
                                                        LPVOID lpInBuf, 
                                                        DWORD nInBufSize, 
                                                        LPVOID lpOutBuf, 
                                                        DWORD nOutBufSize, 
                                                        LPDWORD lpBytesReturned
);

USING_PTYPES

/*
* Get the unique Device Identifier from PPC 2002 - 2003
* see more at http://www.pocketpcdn.com/articles/serial_number2002.html
*
* @param deviceId     : the retrieved device id
*/

void GetSerialNumberFromKernelIoControl(wchar_t* deviceId) {
        
    DWORD dwOutBytes = 0;
    const int nBuffSize = 4096;
    byte arrOutBuff[nBuffSize];
    BOOL bRes = TRUE; 
    
    bRes = KernelIoControl(IOCTL_HAL_GET_DEVICEID, 0, 0, arrOutBuff, nBuffSize, &dwOutBytes);

    if (bRes) {
        string strDeviceInfo = string();
        for (unsigned int i = 0; i<dwOutBytes; i++) {                   
            wchar_t tmp [50];
            wsprintf(tmp, TEXT("%02X"), arrOutBuff[i]);
            strDeviceInfo = strDeviceInfo + tmp;

        }
        del(strDeviceInfo, 0, 40);
        del(strDeviceInfo, 2, 3);
        del(strDeviceInfo, 11, 16);
        del(strDeviceInfo, 17, 4);
        
        wsprintf(deviceId, TEXT("%s"), strDeviceInfo);
            
            
    } else {
        wsprintf(deviceId, TEXT("%s"), TEXT(""));
    }       
        

}

/* 
* Method to write communication settings into device. 
* Used by setConfigurationParameters 
*
* @param rootContext  : the root context node
* @param username     : user name
* @param password     : user password
* @param syncURL      : synchronization url
* @param serverName   : the server uri used to address the sync server
* @param deviceId     : device identifier
* @param errorMsg     : the returned message to vb interface
*/

void saveSyncMLPropsToDM(wchar_t* rootContext, wchar_t* username, wchar_t* password, 
                         wchar_t* syncURL, wchar_t* serverName, wchar_t* deviceId, wchar_t* errorMsg) {
    
    const int VALUESIZE = 512;
    wchar_t context[DIM_MANAGEMENT_PATH];
        
    //
    // Creates the DeviceManager object and extracts the relevant configuration
    // information.
    //
    DeviceManagerFactory factory = DeviceManagerFactory();
    DeviceManager* dm = factory.getDeviceManager();     
    //
    // Writes the modified SyncMnager configuration fields
    //
    wsprintf(context, TEXT("%s/%s"), rootContext, CONTEXT_SPDS_SYNCML);
    
    ManagementNode* spdsConfig = dm->getManagementNode(context);
    if (spdsConfig == NULL) {        
        wsprintf(errorMsg, TEXT("SyncManager configuration not found: %s"), context);
        goto finally;
    }
        
    spdsConfig->setPropertyValue(PROPERTY_SYNC_URL, syncURL);    
    spdsConfig->setPropertyValue(PROPERTY_SERVER_NAME, serverName);
    spdsConfig->setPropertyValue(PROPERTY_USERNAME, username);
    spdsConfig->setPropertyValue(PROPERTY_PASSWORD, password);
    
    // 
    // check if deviceId has arleady a value
    //
    wchar_t previousDeviceId[512];
    spdsConfig->getPropertyValue(PROPERTY_DEVICE_ID, previousDeviceId, 511);     
    
    if (wcscmp(previousDeviceId, TEXT("")) == 0) {
        spdsConfig->setPropertyValue(PROPERTY_DEVICE_ID, deviceId);     
    }

finally:        
    
    wsprintf(errorMsg, TEXT("success"));    
}

/* 
* Method to get communication settings from device. 
* @param name  : name of the property to retrieve value of
* @param value : value of the property
*/

DWORD WINAPI getConfigParameter (wchar_t* name, wchar_t* value) {

    wchar_t rootContext[DIM_MANAGEMENT_PATH];
    wsprintf(rootContext, ROOT_CONTEXT);
        
    wchar_t tmp[512];

    Config* config = NULL;

    config = new Config(rootContext);
    BOOL ret = config->readFromDM();        

    if      (wcscmp(name, SPDM_SYNCURL   ) == 0) {
                wsprintf(tmp, TEXT("%s"),  config->getSyncURL());               
    }
    else if (wcscmp(name, SPDM_SERVERNAME) == 0) {
                wsprintf(tmp, TEXT("%s"),  config->getServerName());
    }
    else if (wcscmp(name, SPDM_PASSWORD  ) == 0) {
                wsprintf(tmp, TEXT("%s"),  config->getPassword());
    }
    else if (wcscmp(name, SPDM_USERNAME  ) == 0) {
                wsprintf(tmp, TEXT("%s"),  config->getUsername());
    }
    else if (wcscmp(name, SPDM_DEVICEID  ) == 0) {
                wsprintf(tmp, TEXT("%s"),  config->getDeviceId());
                if (wcscmp(tmp, TEXT("")) == 0) {
                    GetSerialNumberFromKernelIoControl(tmp);
                    GetSerialNumberFromKernelIoControl(tmp);
                }

    }
    // overwrite the terminator char \0 of wchar_t* array to read value properly

    wcscpy(value, tmp);
    value[wcslen(value)] = L' ';
    
    if (config != NULL)
        delete config;

    return 0;

}

/* 
* Method to set communication settings into device. The parameters are retrieved by Visual Basic interface
*
* @param username     : user name
* @param password     : user password
* @param syncURL      : synchronization url
* @param serverName   : the server uri used to address the sync server
* @param errorMsg     : the returned message to vb interface
*/

DWORD WINAPI setConfigParameters(wchar_t *username, wchar_t* password, 
                                 wchar_t* syncURL, wchar_t* serverName, wchar_t* errorMsg) {
        
    wchar_t rootContext [DIM_MANAGEMENT_PATH];
    wchar_t deviceId [DIM_DEVICE_ID];
    wchar_t res [50];
    
    wsprintf(rootContext, ROOT_CONTEXT);
    GetSerialNumberFromKernelIoControl(deviceId);

    /*
    * Repeat function call GetSerialNumberFromKernelIoControl to fix
    * retrieve serial number from Asus MyPal 620.
    * It's seems this function doesn't work properly if the function
    * is called only one time. 
    */

    GetSerialNumberFromKernelIoControl(deviceId);

    if (wcsstr(syncURL, HTTP_HEADER) == NULL) {
        wchar_t tmp[256];
        wsprintf(tmp, HTTP_HEADER);
        wcscat(tmp, syncURL);
        wsprintf(syncURL, tmp);
    
    }

    saveSyncMLPropsToDM(rootContext, username, password, syncURL, serverName, deviceId, res);       

    wsprintf(errorMsg, TEXT("%s"),  res);   
    
    // overwrite the terminator char \0 of wchar_t* array to read value properly
    errorMsg[wcslen(errorMsg)] = L' ';
    
    return 0;

}

/*
* Method to get the synchronization setting parameters.
* 
* @param sourceName     : source name of which get parameter's value
* @param propertyName   : name of property
* @param propertyValue  : value of property
*/

DWORD WINAPI getSourceParameter (LPWSTR sourceName, LPWSTR propertyName,  LPWSTR propertyValue) {

    wchar_t rootContext [DIM_MANAGEMENT_PATH];
    wsprintf(rootContext, ROOT_CONTEXT);

    Config* config = NULL;

    config = new Config(rootContext);
    BOOL ret = config->readFromDM();
    wchar_t tmp[50];        
    
    SyncSourceConfig syncSourceConfig = SyncSourceConfig();

    if (wcscmp(sourceName, SYNC_APPOINTMENTS) == 0) {                               
            config->getSyncSourceConfig(APPOINTMENTS_NAME, syncSourceConfig);                    
    }
    else if (wcscmp(sourceName, SYNC_CONTACTS) == 0) {
            config->getSyncSourceConfig(CONTACTS_NAME, syncSourceConfig);               
    }
    else if (wcscmp(sourceName, SYNC_TASKS) == 0) {
                config->getSyncSourceConfig(TASKS_NAME, syncSourceConfig);           
        }
    else if (wcscmp(sourceName, SYNC_NOTES) == 0) {
                config->getSyncSourceConfig(NOTES_NAME, syncSourceConfig);
        }
    else if (wcscmp(sourceName, SYNC_FILES) == 0) {
                config->getSyncSourceConfig(FILES_NAME, syncSourceConfig);           
        }
    else if (wcscmp(sourceName, SYNC_FAVORITES) == 0) {
                config->getSyncSourceConfig(FAVORITES_NAME, syncSourceConfig);               
        }
   
    if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_SYNC) == 0) 
        wsprintf(tmp, TEXT("%s"),  syncSourceConfig.getSync(NULL));         
    
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_DIR) == 0) {
        wsprintf(tmp, TEXT("%s"),  syncSourceConfig.getDir(NULL));
        
        wchar_t* loc = wcschr(tmp, L'\\');
        if (loc != NULL)
            wsprintf(tmp, loc);
        else {
            SHGetSpecialFolderPath(NULL, tmp, CSIDL_PERSONAL, FALSE);                

        }
    }  
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_URI) == 0) {
        wsprintf(tmp, TEXT("%s"),  syncSourceConfig.getURI(NULL));                
    }  
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_SYNC_MODES) == 0) {
       
        wsprintf(tmp, TEXT("%s"),  syncSourceConfig.getSyncModes(NULL));                
    } 

    // overwrite the terminator char \0 of wchar_t* array to read value properly
    wcscpy(propertyValue, tmp);
    propertyValue[wcslen(propertyValue)] = L' ';
    
    delete config;
    return 0;

}

/*
* Method to set the synchronization setting parameters.
* 
* @param sourceName     : source name of which get parameter's value
* @param propertyName   : name of property
* @param propertyValue  : value of property
* @param errorMsg       : returned error message
*/

DWORD WINAPI setSourceParameters (wchar_t* sourceName, wchar_t* propertyName, 
                                  wchar_t* propertyValue, wchar_t* errorMsg) {

    wchar_t rootContext [DIM_MANAGEMENT_PATH];
    wsprintf(rootContext, ROOT_CONTEXT);
    
    Config* config = NULL;

    config = new Config(rootContext);
    BOOL ret = config->readFromDM();        

    SyncSourceConfig syncSourceConfig = SyncSourceConfig();

    if (wcscmp(sourceName, SYNC_APPOINTMENTS) == 0) {                                               
    config->getSyncSourceConfig(APPOINTMENTS_NAME, syncSourceConfig);       
    }
    else if (wcscmp(sourceName, SYNC_CONTACTS) == 0) {      
            config->getSyncSourceConfig(CONTACTS_NAME, syncSourceConfig);                
    }
    else if (wcscmp(sourceName, SYNC_TASKS) == 0) {     
            config->getSyncSourceConfig(TASKS_NAME, syncSourceConfig);        
    }
    else if (wcscmp(sourceName, SYNC_NOTES) == 0) {     
            config->getSyncSourceConfig(NOTES_NAME, syncSourceConfig);           
    }
    else if (wcscmp(sourceName, SYNC_FILES) == 0) {
            config->getSyncSourceConfig(FILES_NAME, syncSourceConfig);           
    }
    else if (wcscmp(sourceName, SYNC_FAVORITES) == 0) { 
            config->getSyncSourceConfig(FAVORITES_NAME, syncSourceConfig);       
    }

    if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_SYNC) == 0)             
                    syncSourceConfig.setSync(propertyValue);
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_DIR) == 0) {

        wchar_t t [256];
        wsprintf(t, propertyValue);

        wchar_t* loc = wcschr(t, L'\\');
        if (loc != NULL)
            wsprintf(t, loc);
        else
            wsprintf(t, TEXT("\\"));
        syncSourceConfig.setDir(t);
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_URI) == 0) {
        
        syncSourceConfig.setURI(propertyValue);
    }

    config->setSyncSourceConfig(syncSourceConfig);

    config->saveToDM();
            
    wsprintf(errorMsg, TEXT("success"));
    errorMsg[wcslen(errorMsg)] = L' ';
    
    if (config != NULL)
        delete config;

    return 0;

}

DWORD WINAPI setClientConfiguration(wchar_t* sourceName, wchar_t* propertyName, 
                                     wchar_t* propertyValue, wchar_t* errorMsg)  {
    
    const int VALUESIZE = 512;
    wchar_t context[DIM_MANAGEMENT_PATH];
        
    //
    // Creates the DeviceManager object and extracts the relevant configuration
    // information.
    //
    DeviceManagerFactory factory = DeviceManagerFactory();
    DeviceManager* dm = factory.getDeviceManager();     
    //
    // Writes the modified SyncMnager configuration fields
    //
    wsprintf(context, TEXT("%s/%s"), ROOT_CONTEXT, CONTEXT_SPDS_SOURCESPROP);
    
    ManagementNode* spdsConfig = dm->getManagementNode(context);
    if (spdsConfig == NULL) {        
        wsprintf(errorMsg, TEXT("SyncManager configuration not found: %s"), context);
        goto finally;
    }
    
    if (sourceName == NULL || wcscmp(sourceName, TEXT("")) == 0) {
        
        if (wcscmp(propertyName, PROPERTY_SOURCEPROP_SPDM_LOG_LEVEL) == 0)
            spdsConfig->setPropertyValue(PROPERTY_SOURCEPROP_LOG_LEVEL, propertyValue);
                                   
    }
finally:
    return 0;
}

/*
* The method recall the other getClientConfiguration and set the variable isFromDB as true.
* This variable means that the caller is the VB and so it's necessary remove the final char '\0' 
* to permit the vb to understand the return string properly
*/

DWORD WINAPI getClientConfiguration (wchar_t* sourceName, wchar_t* propertyName, 
                                     wchar_t* propertyValue) {   

    return getClientConfigurationInternal (sourceName, propertyName, propertyValue, TRUE);
}

/*
* Method to get the client configurations. These property are set by installing process.
* These are extra parameters not used in synchronization process 
* but are needed for example to show or hide some fields in GUI interface.
* 
* @param sourceName     : source name of which get parameter's value
* @param propertyName   : name of property
* @param propertyValue  : value of property
*/

DWORD getClientConfigurationInternal (wchar_t* sourceName, wchar_t* propertyName, 
                                     wchar_t* propertyValue, BOOL isFromVB) {    

    const int VALUESIZE = 512;
    int nc = 0;
    wchar_t context[DIM_MANAGEMENT_PATH];
    wchar_t s [DIM_MANAGEMENT_PATH];
        
    
    wchar_t buf[VALUESIZE]; 

    DeviceManagerFactory factory = DeviceManagerFactory();
    DeviceManager* dm = factory.getDeviceManager();     
    
    wsprintf(context, TEXT("%s/%s"), ROOT_CONTEXT, CONTEXT_SPDS_SOURCESPROP);
    
    ManagementNode* spdsConfig = dm->getManagementNode(context);
    
    if (spdsConfig == NULL) {
        lastErrorCode = ERR_INVALID_CONTEXT;
        wsprintf(lastErrorMsg, TEXT("SyncManager configuration not found: %s"), context);
        goto finally;
    }
    
    /*
    * if sourceName is empty then is a property of root context
    */
    if (sourceName == NULL || wcscmp(sourceName, TEXT("")) == 0) {
        
        if (wcscmp(propertyName, PROPERTY_SOURCEPROP_SPDM_REMOTE_SETTINGS) == 0)
            spdsConfig->getPropertyValue(PROPERTY_SOURCEPROP_REMOTE_SETTINGS, buf, VALUESIZE); 
        else if (wcscmp(propertyName, PROPERTY_SOURCEPROP_SPDM_LOG_LEVEL) == 0)
            spdsConfig->getPropertyValue(PROPERTY_SOURCEPROP_LOG_LEVEL, buf, VALUESIZE); 
        else
            wsprintf(buf, TEXT(""));
                   
        wcscpy(propertyValue, buf);                                
        
        if (isFromVB != NULL && isFromVB == TRUE)
            propertyValue[wcslen(propertyValue)] = L' ';
        
        return 0;
    }

    nc = spdsConfig->getChildrenCount();
        
    if (nc > 0) {
        lastErrorCode = 0;
        ManagementNode** children = new ManagementNode*[nc];

        spdsConfig->getChildren(children, &nc);
        if (lastErrorCode) {
            goto finally;
        }
        
        for (int i = 0; i < nc; ++i) {
                        
            if (wcscmp(sourceName, SYNC_APPOINTMENTS) == 0) {
                wsprintf(s, TEXT("%s"), APPOINTMENTS_NAME);
            }
            else if (wcscmp(sourceName, SYNC_CONTACTS) == 0) {
                wsprintf(s, TEXT("%s"), CONTACTS_NAME);
            }
            else if (wcscmp(sourceName, SYNC_TASKS) == 0) {
                wsprintf(s, TEXT("%s"), TASKS_NAME);
            }
            else if (wcscmp(sourceName, SYNC_FILES) == 0) {
                wsprintf(s, TEXT("%s"), FILES_NAME);
            }
            else if (wcscmp(sourceName, SYNC_NOTES) == 0) {
                wsprintf(s, TEXT("%s"), NOTES_NAME);
            }
            else if (wcscmp(sourceName, SYNC_FAVORITES) == 0) {
                wsprintf(s, TEXT("%s"), FAVORITES_NAME);
            }
                        
            children[i]->getPropertyValue(PROPERTY_SOURCE_NAME, buf, VALUESIZE); 
            
            if (wcscmp(s, buf) == 0) {
                
                if (wcscmp(propertyName, PROPERTY_SOURCEPROP_SPDM_SHOW) == 0) {
                    
                    children[i]->getPropertyValue(PROPERTY_SOURCEPROP_SHOW, buf, VALUESIZE); 
                }
                else {
                    wsprintf(buf, TEXT(""));                   
                }

                wcscpy(propertyValue, buf);                                
                propertyValue[wcslen(propertyValue)] = L' ';

                delete children[i];
                
                break;
            }
            
            delete children[i];

        }
       
        delete [] children;
    }
    
finally:
    
    return 0;
    
}
