/*
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//
// @author Marco Magistrali
//

#include "pim/TaskBuilder.h"

/*
* Method to populate the task XML structure. 
* It get every property name and value and build the XML structure to exchange with server.
*
* @param taskStringItem : the string that will contain the xml structure
* @param pTask          : the task object that contain every value
*
*/

void populateTaskStringItem (string &taskStringItem, ITask *pTask) {

    BSTR element = NULL;
    long elementLong = 0;
    DATE date = NULL;
    string element_s = string();

    VARIANT_BOOL variantBool = NULL; // 0==false -1==true
    wchar_t tempDate [DIM_LOCAL_TEMP];

    wchar_t* charBlob = NULL;
        
    taskStringItem = TEXT("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
    taskStringItem = taskStringItem + TEXT("<task>");
            
        // BSTR
        pTask->get_Subject(&element);
        element_s = string(element); encodeSpecialChar(element_s);
        taskStringItem  = taskStringItem + TEXT("<Subject>") + element_s + TEXT("</Subject>");
                    
        pTask->get_Categories(&element);
        element_s = string(element); encodeSpecialChar(element_s);
        taskStringItem  = taskStringItem + TEXT("<Categories>") + element_s + TEXT("</Categories>");

        pTask->get_ReminderSoundFile(&element);
        element_s = string(element); encodeSpecialChar(element_s);
        taskStringItem  = taskStringItem + TEXT("<ReminderSoundFile>") + element_s + TEXT("</ReminderSoundFile>");
    
        pTask->get_Body(&element);
        element_s = string(element); encodeSpecialChar(element_s);
        taskStringItem  = taskStringItem + TEXT("<Body>") + element_s + TEXT("</Body>");                  
    
                

        // VARIANT_BOOL
        pTask->get_ReminderSet(&variantBool);
        wsprintf(tempDate, TEXT("%i"), variantBool);
        normalizeBoolean(tempDate);
        taskStringItem  = taskStringItem + TEXT("<ReminderSet>") + tempDate + TEXT("</ReminderSet>");   

        pTask->get_Complete(&variantBool);
        wsprintf(tempDate, TEXT("%i"), variantBool);
        normalizeBoolean(tempDate);
        taskStringItem  = taskStringItem + TEXT("<Complete>") + tempDate + TEXT("</Complete>"); 

        pTask->get_TeamTask(&variantBool);
        wsprintf(tempDate, TEXT("%i"), variantBool);
        normalizeBoolean(tempDate);
        taskStringItem  = taskStringItem + TEXT("<TeamTask>") + tempDate + TEXT("</TeamTask>"); 
                
                        
        // DATE
        pTask->get_StartDate(&date);

        wsprintf(tempDate, TEXT(""));           
        double toCompare = MAX_DATE_DOUBLE;
        
        if (date != toCompare) { 
                doubleToSystemTime(tempDate, date);

        }
        taskStringItem  = taskStringItem + TEXT("<StartDate>") + tempDate + TEXT("</StartDate>");               


        pTask->get_DueDate(&date);

        wsprintf(tempDate, TEXT(""));
        if (date != toCompare) { 
                doubleToSystemTime(tempDate, date);
        }
        taskStringItem  = taskStringItem + TEXT("<DueDate>") + tempDate + TEXT("</DueDate>");
        
        /*
        pTask->get_DateCompleted(&date);

        wsprintf(tempDate, TEXT(""));
        if (date != toCompare) { 
                doubleToSystemTime(tempDate, date);
        }
        taskStringItem  = taskStringItem + TEXT("<DateCompleted>") + tempDate + TEXT("</DateCompleted>");
        */

        pTask->get_ReminderTime(&date);

        wsprintf(tempDate, TEXT(""));
        if (date != toCompare) { 
                doubleToSystemTime(tempDate, date);
        }
        taskStringItem  = taskStringItem + TEXT("<ReminderTime>") + tempDate + TEXT("</ReminderTime>");
        

        // long 
        
        pTask->get_Sensitivity(&elementLong);           
        wsprintf(tempDate, TEXT("%i"), elementLong);
        taskStringItem  = taskStringItem + TEXT("<Sensitivity>") + tempDate + TEXT("</Sensitivity>");
        
        pTask->get_ReminderOptions(&elementLong);               
        wsprintf(tempDate, TEXT("%i"), elementLong);
        taskStringItem  = taskStringItem + TEXT("<ReminderOptions>") + tempDate + TEXT("</ReminderOptions>");
                
        pTask->get_Importance(&elementLong);            
        wsprintf(tempDate, TEXT("%i"), elementLong);
        taskStringItem  = taskStringItem + TEXT("<Importance>") + tempDate + TEXT("</Importance>");

        /******************************************************/
        //Recurrence
        pTask->get_IsRecurring(&variantBool);           
        wsprintf(tempDate, TEXT("%i"), variantBool);
        normalizeBoolean(tempDate);
        taskStringItem  = taskStringItem + TEXT("<IsRecurring>") + tempDate + TEXT("</IsRecurring>");
        
        IRecurrencePattern *pRecurrence;
        pTask->GetRecurrencePattern (&pRecurrence);

        pRecurrence->get_RecurrenceType(&elementLong);          
        wsprintf(tempDate, TEXT("%i"), elementLong);
        taskStringItem  = taskStringItem + TEXT("<RecurrenceType>") + tempDate + TEXT("</RecurrenceType>");

        pRecurrence->get_Interval(&elementLong);                
        wsprintf(tempDate, TEXT("%i"), elementLong);
        taskStringItem  = taskStringItem + TEXT("<Interval>") + tempDate + TEXT("</Interval>");

        pRecurrence->get_MonthOfYear(&elementLong);             
        wsprintf(tempDate, TEXT("%i"), elementLong);
        taskStringItem  = taskStringItem + TEXT("<MonthOfYear>") + tempDate + TEXT("</MonthOfYear>");

        pRecurrence->get_DayOfMonth(&elementLong);              
        wsprintf(tempDate, TEXT("%i"), elementLong);
        taskStringItem  = taskStringItem + TEXT("<DayOfMonth>") + tempDate + TEXT("</DayOfMonth>");
        
        pRecurrence->get_DayOfWeekMask(&elementLong);           
        wsprintf(tempDate, TEXT("%i"), elementLong);
        taskStringItem  = taskStringItem + TEXT("<DayOfWeekMask>") + tempDate + TEXT("</DayOfWeekMask>");

        pRecurrence->get_Instance(&elementLong);                
        wsprintf(tempDate, TEXT("%i"), elementLong);
        taskStringItem  = taskStringItem + TEXT("<Instance>") + tempDate + TEXT("</Instance>");

        pRecurrence->get_PatternStartDate(&date);

        wsprintf(tempDate, TEXT(""));           
        toCompare = MAX_DATE_DOUBLE;
        
        if (date != toCompare) { 
                doubleToSystemTime(tempDate, date);                     
        }
        taskStringItem  = taskStringItem + TEXT("<PatternStartDate>") + tempDate + TEXT("</PatternStartDate>");         
        
        pRecurrence->get_NoEndDate(&variantBool);
        wsprintf(tempDate, TEXT("%i"), variantBool);
        normalizeBoolean(tempDate);
        taskStringItem  = taskStringItem + TEXT("<NoEndDate>") + tempDate + TEXT("</NoEndDate>");       

        pRecurrence->get_PatternEndDate(&date);

        wsprintf(tempDate, TEXT(""));           
        toCompare = MAX_DATE_DOUBLE;
        
        if (date != toCompare) { 
                doubleToSystemTime(tempDate, date);             
        }
        taskStringItem  = taskStringItem + TEXT("<PatternEndDate>") + tempDate + TEXT("</PatternEndDate>");             
    
    taskStringItem = taskStringItem +  TEXT("</task>");
            
    if (dbg)
            MessageBox (NULL, taskStringItem, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);

}


/*
* Method that complete an task object retrieving property value by an XML structure.
* It parse the XML structure, get the every property value using its name and set it into the object.
*
* @param pTask      : the task object that contain every value
* @param ptrData    : the variable that contain the xml structure
*/

void completeTask  (ITask *pTask, wchar_t * ptrData) {

    wchar_t* dummy = NULL;
    wchar_t* dummyEvent = NULL;             
    string   dummyEvent_s           = string();
    pTask->ClearRecurrencePattern ();
    
    //VARIANT_BOOL -- ReminderSet 
    dummyEvent = getElementContent(ptrData, TEXT ("ReminderSet"), NULL);            
    if (dummyEvent != NULL && (wcscmp(dummyEvent, TEXT("-1")) == 0 || wcscmp(dummyEvent, TEXT("True")) == 0)) {
        pTask->put_ReminderSet(VARIANT_TRUE);
    }
    else {
        pTask->put_ReminderSet(VARIANT_FALSE);  
    }               
    if (dummyEvent != NULL)
    delete dummyEvent;
    dummyEvent=NULL;            
    
    dummyEvent = getElementContent(ptrData, TEXT ("Complete"), NULL);           
    if (dummyEvent != NULL && (wcscmp(dummyEvent, TEXT("-1")) == 0 || wcscmp(dummyEvent, TEXT("True")) == 0)) {
        pTask->put_Complete(VARIANT_TRUE);
    }
    else {
        pTask->put_Complete(VARIANT_FALSE);     
    }

    if (dummyEvent != NULL)
        delete dummyEvent;dummyEvent=NULL;      
    
    dummyEvent = getElementContent(ptrData, TEXT ("TeamTask"), NULL);           
    if (dummyEvent != NULL && (wcscmp(dummyEvent, TEXT("-1")) == 0 || wcscmp(dummyEvent, TEXT("True")) == 0)) {
        pTask->put_TeamTask(VARIANT_TRUE);
    }
    else {
        pTask->put_TeamTask(VARIANT_FALSE);     
    }
    if (dummyEvent != NULL)
        delete dummyEvent; dummyEvent=NULL;

        
    // BSTR
    dummyEvent = getElementContent(ptrData, TEXT ("Subject"), NULL);
    if (dummyEvent != NULL) {
        dummyEvent_s = string(dummyEvent); decodeSpecialChar(dummyEvent_s);
        pTask->put_Subject ((wchar_t*)dummyEvent_s);
        delete dummyEvent;  dummyEvent=NULL;
    }
        
    dummyEvent = getElementContent(ptrData, TEXT ("Categories"), NULL);
    if (dummyEvent != NULL) {
        dummyEvent_s = string(dummyEvent); decodeSpecialChar(dummyEvent_s);
        pTask->put_Categories ((wchar_t*)dummyEvent_s);
        delete dummyEvent; dummyEvent=NULL;
    }
    
    dummyEvent = getElementContent(ptrData, TEXT ("ReminderSoundFile"), NULL);
    if (dummyEvent != NULL) {
        dummyEvent_s = string(dummyEvent); decodeSpecialChar(dummyEvent_s);
        pTask->put_ReminderSoundFile  ((wchar_t*)dummyEvent_s);
        delete dummyEvent; dummyEvent=NULL;
    }
    
    dummyEvent = getElementContent(ptrData, TEXT ("Body"), NULL);
    if (dummyEvent != NULL) {
        dummyEvent_s = string(dummyEvent); decodeSpecialChar(dummyEvent_s);
        pTask->put_Body  ((wchar_t*)dummyEvent_s);   
        delete dummyEvent;  dummyEvent=NULL;                                    
    }   

    wchar_t localTemp[DIM_LOCAL_TEMP];
    //DATE
    dummyEvent = getElementContent(ptrData, TEXT ("StartDate"), NULL);
    if (dummyEvent == NULL || (wcscmp(dummyEvent, TEXT("")) == 0))
        ;
    else {
        DATE st;
        systemTimeToDouble(dummyEvent, &st, NULL);
        if (dbg) {
                wsprintf(localTemp, TEXT("st %f"), st); 
                MessageBox (NULL, localTemp, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
        }
    pTask->put_StartDate(st);
    delete dummyEvent;
    }
    
    dummyEvent = getElementContent(ptrData, TEXT ("DueDate"), NULL);
    if (dummyEvent == NULL || (wcscmp(dummyEvent, TEXT("")) == 0))
        ;
    else {
        DATE dd;
        systemTimeToDouble(dummyEvent, &dd, NULL);
        if (dbg) {
                wsprintf(localTemp, TEXT("dd %f"), dd); 
                MessageBox (NULL, localTemp, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
        }
    pTask->put_DueDate(dd);
    delete dummyEvent;
    }



        //long
    dummyEvent = getElementContent(ptrData, TEXT ("Importance"), NULL);
    if (dummyEvent != NULL) {
        pTask->put_Importance(wcstol(dummyEvent, &dummy, 0));    
        delete dummyEvent; dummyEvent=NULL;
    }

    dummyEvent = getElementContent(ptrData, TEXT ("Sensitivity"), NULL);
    if (dummyEvent != NULL) {
        pTask->put_Sensitivity(wcstol(dummyEvent, &dummy, 0));    
        delete dummyEvent; dummyEvent=NULL;
    }


    dummyEvent = getElementContent(ptrData, TEXT ("ReminderOptions"), NULL);
    if (dummyEvent != NULL) {
        pTask->put_ReminderOptions (wcstol(dummyEvent, &dummy, 0));
        delete dummyEvent; dummyEvent=NULL;
    }
        
    /****************************************************/
    
    //Recurrence
    dummyEvent = getElementContent(ptrData, TEXT ("IsRecurring"), NULL);
    
    if (dbg) {                      
        MessageBox (NULL, dummyEvent, TEXT ("Is Task Recurring"), MB_SETFOREGROUND |MB_OK);
    }

    if (dummyEvent != NULL && (wcscmp(dummyEvent, TEXT("-1")) == 0 
        || wcscmp(dummyEvent, TEXT("True")) == 0)) {                    
        
        IRecurrencePattern *pRecurrence;
        pTask->GetRecurrencePattern (&pRecurrence);

        dummyEvent = getElementContent(ptrData, TEXT ("RecurrenceType"), NULL);
        if (dummyEvent != NULL && wcstol(dummyEvent, &dummy, 0) >= 0) {
            pRecurrence->put_RecurrenceType(wcstol(dummyEvent, &dummy, 0));                 
            delete dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("Interval"), NULL);       
        if (dummyEvent != NULL && wcstol(dummyEvent, &dummy, 0) >= 0) {
            pRecurrence->put_Interval(wcstol(dummyEvent, &dummy, 0));               
            delete dummyEvent;dummyEvent=NULL;
        }
            
        dummyEvent = getElementContent(ptrData, TEXT ("MonthOfYear"), NULL);
        if (dummyEvent != NULL && wcstol(dummyEvent, &dummy, 0) > 0) {
            pRecurrence->put_MonthOfYear(wcstol(dummyEvent, &dummy, 0));                    
            delete dummyEvent;dummyEvent=NULL;
        }
                
        dummyEvent = getElementContent(ptrData, TEXT ("DayOfMonth"), NULL);
        if (dummyEvent != NULL && wcstol(dummyEvent, &dummy, 0) > 0) {
            pRecurrence->put_DayOfMonth(wcstol(dummyEvent, &dummy, 0));                     
            delete dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("DayOfWeekMask"), NULL);
        if (dummyEvent != NULL && wcstol(dummyEvent, &dummy, 0) > 0) {
            pRecurrence->put_DayOfWeekMask(wcstol(dummyEvent, &dummy, 0));                  
            delete dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("Instance"), NULL);
        if (dummyEvent != NULL && wcstol(dummyEvent, &dummy, 0) > 0) {
            pRecurrence->put_Instance(wcstol(dummyEvent, &dummy, 0));                       
            delete dummyEvent;dummyEvent=NULL;
        }

                
        //Pattern Start Date
        dummyEvent = getElementContent(ptrData, TEXT ("PatternStartDate"), NULL);

        if (dummyEvent == NULL || (wcscmp(dummyEvent, TEXT("")) == 0))
            ;
        else {
            DATE st;
            systemTimeToDouble(dummyEvent, &st, NULL);

            pRecurrence->put_PatternStartDate(st);
            delete dummyEvent;
        }

        //VARIANT_BOOL -- NoEndDate
        dummyEvent = getElementContent(ptrData, TEXT ("NoEndDate"), NULL);
        

        if (dummyEvent != NULL && (wcscmp(dummyEvent, TEXT("-1")) == 0 || 
            wcscmp(dummyEvent, TEXT("True")) == 0)) {
            pRecurrence->put_NoEndDate(VARIANT_TRUE);

        } else {
            pRecurrence->put_NoEndDate(VARIANT_FALSE);

            //Pattern End Date
            dummyEvent = getElementContent(ptrData, TEXT ("PatternEndDate"), NULL);

            if (dummyEvent == NULL || (wcscmp(dummyEvent, TEXT("")) == 0))
                ;
            else {
                DATE en;
                systemTimeToDouble(dummyEvent, &en, NULL);

                pRecurrence->put_PatternEndDate(en);
                delete dummyEvent;
            }

        }

    } else {
            ;// task is not recurring...
    }

}

/*
* The method calculate the CRC hash value for the task item.
* It create the XML structure and calculate the crc value.
* Use to compare for understand the modified value!
*
* @param pTask : the task object that contain every value
*/

long calculateTaskHash (ITask *pTask) {

    CCrc32 cc = CCrc32();
    string p = string();
    BSTR element = NULL;    

    populateTaskStringItem(p, pTask);       
    
    BSTR dummy = p;

    long l = cc.crc32String( &dummy);
        
    return l;
}
