/*
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//
// @author Marco Magistrali
//

#include <stdio.h>
#include <windows.h>
#include <Winbase.h>
#include "common/Log.h"

#include "pim/Utils.h"
#include "pim/ActionLog.h"
#include "pim/base64.h"
#include "pim/SettingFunctions.h"

#include <tchar.h>

// DST conversion items
#define IS_LEAP_YEAR(y) (((y) & 3) == 0)

#define BASE_DOW          4                  // 1/1/1970 was a Thursday.
#define SECONDS_IN_A_DAY  (24L * 60L * 60L)  // Number of seconds in one day.

// Month to Year Day conversion array.
int M2YD[] = {
   0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365
};
// Month to Leap Year Day conversion array.
int M2LYD[] = {
   0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366
};

USING_PTYPES


/*
* The method writes error code and error into log file.
*/

void error() {
    wsprintf(logmsg, TEXT("Error code: %d\nerror message: %s"), lastErrorCode, lastErrorMsg);
    LOG.error(logmsg);
}

/*
* The method writes error code and error into variable that will be send to the VB interface.
*
* @param errorMessage : variable to share
*/

void error(wchar_t* errorMessage) {
    wsprintf(errorMessage, TEXT("Error code: %d\nerror message: %s"), lastErrorCode, lastErrorMsg);    
        ActionLOG.action(errorMessage);
}


/*
 * The method extracts the content of a tag into a XML message. It is supposed the
 * message is a valid XML message. It returns NULL in case the tag is not
 * found or the XML fragment is not in the expected form.
 * The returned pointer (if not NULL) is allocated with the new operator and
 * must be discarded with the operator delete.
 *
 * @param xml : the xml fragment
 * @param tag : the tag we want the content
 * @param pos : (OUTPUT) the position just after the closing tag (ignored if NULL)
 *
 */
wchar_t* getElementContent(wchar_t* xml, wchar_t* tag, unsigned int* pos) {
    wchar_t* p1       = NULL;
    wchar_t* p2       = NULL;
    wchar_t* ret      = NULL;
    BOOL charFound    = FALSE;

    wchar_t openTag[50];
    wchar_t closeTag[50];

    unsigned int xmlLength = wcslen(xml);
    unsigned int l = wcslen(tag);

    if (pos != NULL) {
        *pos = 0;
    }

    wsprintf(openTag, TEXT("<%s>"), tag);
    wsprintf(closeTag, TEXT("</%s>"), tag);

    p1 = wcsstr(xml, openTag);

    if (p1 == NULL) { // tag can have attributes
        //
        // This is abcxyz
        //
        //goto finally;

        // if p1 is null I try to discover the next '>' char to close the tag. If does not exist 
        // return NULL
        
        // try to find "<tagName/>". If found it return null.
        wsprintf(openTag, TEXT("<%s/>"), tag);
        p1 = wcsstr(xml, openTag);

        if (p1 != NULL) {
            goto finally;
        }
        
        // try to find "<tagName"
        wsprintf(openTag, TEXT("<%s"), tag);
        p1 = wcsstr(xml, openTag);

        if (p1 == NULL) {
            goto finally;
        }
        
        p1 = p1 + l + 1;   // <body_ 

        for (unsigned int k = 0; k < xmlLength; k++) { // Suppose max length as the xml string
            p1 = p1 + 1;
            if (*p1 == 0) {
                goto finally;
            }
            else if (*p1 == '>') {
                charFound = TRUE;
                p1 = p1 + 1;
                break;
            }            
        }
        if (!charFound)
            goto finally;
        
        
    } else {  // tag doesn't have attribute. Original version

        p1 = p1+l+2;
        
    }
    
    if (*p1 == 0) {
        //
        // This is abc<tag>\0
        //
        goto finally;
    }

    p2 = wcsstr(p1, closeTag);

    if (p2 == NULL) {
        //
        // This is abc<tag>xyz\0
        //
        goto finally;
    }

    ret = new wchar_t[p2-p1+1];

    wcsncpy(ret, p1, p2-p1);
    ret[p2-p1] = 0;

    if (pos != NULL) {
        *pos = p2-xml+l+3;
    }
   
    finally:

    return ret;

}
/*
 * The below old method only discover <tag> token.
 * The new getElementContent method discover <tag attribute="1" ...> token
 *
 *
wchar_t* getElementContent(wchar_t* xml, wchar_t* tag, unsigned int* pos) {
    wchar_t* p1       = NULL;
    wchar_t* p2       = NULL;
    wchar_t* ret      = NULL;

    wchar_t openTag[50];
    wchar_t closeTag[50];

    unsigned int l = wcslen(tag);

    if (pos != NULL) {
        *pos = 0;
    }

    wsprintf(openTag, TEXT("<%s>"), tag);
    wsprintf(closeTag, TEXT("</%s>"), tag);

    p1 = wcsstr(xml, openTag);

    if (p1 == NULL) {
        //
        // This is abcxyz
        //
        goto finally;
    }
    p1 = p1+l+2;

    if (*p1 == 0) {
        //
        // This is abc<tag>\0
        //
        goto finally;
    }

    p2 = wcsstr(p1, closeTag);

    if (p2 == NULL) {
        //
        // This is abc<tag>xyz\0
        //
        goto finally;
    }

    ret = new wchar_t[p2-p1+1];

    wcsncpy(ret, p1, p2-p1);
    ret[p2-p1] = 0;

    if (pos != NULL) {
        *pos = p2-xml+l+3;
    }
   
    finally:

    return ret;

}
*/
/*
* The method is used to convert BOOL type in the same string type
*
* @param localTemp : the variable that contain old and new value 
*/

void normalizeBoolean(wchar_t* localTemp) {
        
    if (wcscmp(localTemp, TEXT("-1")) == 0) {
        wsprintf(localTemp, TEXT("True"));
    } else {
        wsprintf(localTemp, TEXT("False"));
    }
}

/*
* The method replaces &lt; or &gt; with < or > in server response syncML message. 
* 
* @param s : the string to be replaced
*/
void replaceAmpLGt(string &s) {
        
    int position = 0;
    
    // search for &lt;
    position = pos(TEXT("&lt;"), s);
    while (position != -1) {
        del(s, position, 4);
        ins(TEXT("<"), s, position);
        position = pos(TEXT("&lt;"), s);
    }
    // now search for &lt;
    position = 0;
    position = pos(TEXT("&gt;"), s);
    while (position != -1) {
        del(s, position, 4);
        ins(TEXT(">"), s, position);
        position = pos(TEXT("&gt;"), s);
    }
    
}

/*
* The method removes \n from the start if there is.
* The start 1n was a bug that has been fixed in new SyncClient API version.
* It works on original array to save memory.
*
* @param ptr : the array to be modify
*/

void removeStartCarriage(wchar_t** ptr) {
   
    // if start with CRLF, 
    if ( ptr != NULL &&
        *ptr != NULL &&
        (int)((*ptr)[0]) == 13 && (int)((*ptr)[1]) == 10)
        *ptr = &((*ptr)[2]);


}

/*
* The method removes \n from the end if there is.
* This method is used to remove carriage\return in reading line from file notes.dat, 
* briefcase.dat and favorites.dat.
* So it will be possible compare properly file names retrived by file system
* and these file name
*
* @param ptr : the filename ended with \n
*/

void removeEndCarriage(wchar_t** ptr) {
    int length = 0;
    if (*ptr != NULL)
        length = wcslen(*ptr);
    
            
    // if start with CRLF, 
    if ( ptr != NULL &&
        *ptr != NULL &&
        (int)((*ptr)[length -1]) == 10)
        (*ptr)[length -1] = 0;


}

void encodeSpecialChar(string &s) {
        
    int position = 0, previousPosition = 0;
       
    //
    // &
    //
    position = pos(TEXT("&"), s);
    while (position != -1) {        
        ins(TEXT("&amp;"), s, position);
        del(s, position + 5, 1);        
        position = position + 2;
        previousPosition = position;                
        position = pos(TEXT("&"), &s[position]);
        
        if (position == -1) {
            break;
        }
        position = previousPosition + position;
    }  

    //
    // <
    //    
    position = 0;
    previousPosition = 0;
    position = pos(TEXT("<"), s);
    while (position != -1) {        
        ins(TEXT("&lt;"), s, position);
        del(s, position + 4, 1);        
        position = position + 2;
        previousPosition = position;                
        position = pos(TEXT("<"), &s[position]);
        
        if (position == -1) {
            break;
        }
        position = previousPosition + position;
    }  

    //
    // >
    //    
    position = 0;
    previousPosition = 0;
    position = pos(TEXT(">"), s);
    while (position != -1) {        
        ins(TEXT("&gt;"), s, position);
        del(s, position + 4, 1);        
        position = position + 2;
        previousPosition = position;                
        position = pos(TEXT(">"), &s[position]);
        
        if (position == -1) {
            break;
        }
        position = previousPosition + position;
    } 
      
}


void decodeSpecialChar(string &s) {
        

    int position = 0, previousPosition = 0;
       
    //
    // &
    //
    position = pos(TEXT("&amp;"), s);
    while (position != -1) {        
        ins(TEXT("&"), s, position);
        del(s, position + 1, 5);                
        previousPosition = position;                
        position = pos(TEXT("&amp;"), &s[position]);
        
        if (position == -1) {
            break;
        }
        position = previousPosition + position;
    }  
    
    //
    // <
    //
    position = 0;
    previousPosition = 0;
    position = pos(TEXT("&lt;"), s);
    while (position != -1) {        
        ins(TEXT("<"), s, position); 
        del(s, position + 1, 4);               
        previousPosition = position;                
        position = pos(TEXT("&lt;"), &s[position]);
        
        if (position == -1) {
            break;
        }
        position = previousPosition + position;
    }  
    
    //
    // >
    //
    position = 0;
    previousPosition = 0;
    position = pos(TEXT("&gt;"), s);
    while (position != -1) {        
        ins(TEXT(">"), s, position);
        del(s, position + 1, 4);                
        previousPosition = position;                
        position = pos(TEXT("&gt;"), &s[position]);
        
        if (position == -1) {
            break;
        }
        position = previousPosition + position;
    } 
      
}

/*
* The method return current system time in VariantTime format 
* used in FILEFileManagement
*/

DATE getSystemTime() {

    SYSTEMTIME t;
    DATE date;
    BOOL isUTC = TRUE;
    wchar_t localTemp [DIM_LOCAL_TEMP];
    
    GetLocalTime (&t);
    
    // conversion from actual time to UTC
    // if propertyValue PROPERTY_SOURCEPROP_SYSTEMTIME_TO_LOCALTIME == 1 then 
    //wchar_t propertyValue[10];
    //getClientConfigurationInternal (NULL, PROPERTY_SOURCEPROP_SPDM_SYSTEMTIME_TO_LOCALTIME, propertyValue, FALSE); 
    
    //if (wcscmp(propertyValue, TEXT("1")) == 0) {
    //    isUTC = TRUE;    
    //}
    
    if (isUTC) {
        localTimeToUTC(t);
    }

    wchar_t month [10];
    wchar_t day   [10];
    wchar_t hour  [10];
    wchar_t minute[10];
    wchar_t second[10];
    
    // month
    if (t.wMonth < 10)
        wsprintf(month, TEXT("0%i"), t.wMonth);
    else
        wsprintf(month, TEXT("%i"), t.wMonth);
    
    // day
    if (t.wDay  < 10)
        wsprintf(day, TEXT("0%i"), t.wDay);
    else
        wsprintf(day, TEXT("%i"), t.wDay);
    
    // hour
    if (t.wHour  < 10)
        wsprintf(hour, TEXT("0%i"), t.wHour);
    else
        wsprintf(hour, TEXT("%i"), t.wHour);

    // minute
    if (t.wMinute < 10)
        wsprintf(minute, TEXT("0%i"), t.wMinute);
    else
        wsprintf(minute, TEXT("%i"), t.wMinute);

    // second
    if (t.wSecond < 10)
        wsprintf(second, TEXT("0%i"), t.wSecond);
    else
        wsprintf(second, TEXT("%i"), t.wSecond);
    
    if (isUTC) {
        wsprintf(localTemp, TEXT("%i%s%sT%s%s%sZ"), t.wYear, month, day, hour, minute, second);
    } else {
        wsprintf(localTemp, TEXT("%i%s%sT%s%s%s"), t.wYear, month, day, hour, minute, second);
    }
    //MessageBox (NULL, localTemp, TEXT ("Update"), MB_SETFOREGROUND |MB_OK); 
    systemTimeToDouble(localTemp, &date, NULL);
    
    return date;

}

/*
* The method return current system time from VariantTime format 
*
* @param tempDate : the variable to write date in
* @param date     : the date in variant time format
*/

void doubleToSystemTime(wchar_t* tempDate, DATE date) {
    doubleToSystemTimeHourZero(tempDate, date, FALSE);
}

/*
* Is like doubleToSystemTime but set hour to 0. It used directly for anniversary and birthday.
* if not the time represented by date is the same in which the date is written 
* on the device
*/
void doubleToSystemTimeHourZero(wchar_t* tempDate, DATE date, BOOL isHourToSetToZero) {
        
    SYSTEMTIME t;
    BOOL isUTC = TRUE;
    VariantTimeToSystemTime(date, &t); 
    
    if (isHourToSetToZero) {
        t.wHour = 0;
        t.wMinute = 0;
        t.wSecond = 0;
    }
    

    // conversion from actual time to UTC    
    //wchar_t propertyValue[10];
    //getClientConfigurationInternal (NULL, PROPERTY_SOURCEPROP_SPDM_SYSTEMTIME_TO_LOCALTIME, propertyValue, FALSE); 

    //if (wcscmp(propertyValue, TEXT("1")) == 0) {
    //    isUTC = TRUE;
    //}
    
    if (isUTC) {
        localTimeToUTC(t);
    }

    wchar_t month [10];
    wchar_t day   [10];
    wchar_t hour  [10];
    wchar_t minute[10];
    wchar_t second[10];
    
    // month
    if (t.wMonth < 10)
        wsprintf(month, TEXT("0%i"), t.wMonth);
    else
        wsprintf(month, TEXT("%i"), t.wMonth);
    
    // day
    if (t.wDay  < 10)
        wsprintf(day, TEXT("0%i"), t.wDay);
    else
        wsprintf(day, TEXT("%i"), t.wDay);
    
    // hour
    if (t.wHour  < 10)
        wsprintf(hour, TEXT("0%i"), t.wHour);
    else
        wsprintf(hour, TEXT("%i"), t.wHour);

    // minute
    if (t.wMinute < 10)
        wsprintf(minute, TEXT("0%i"), t.wMinute);
    else
        wsprintf(minute, TEXT("%i"), t.wMinute);

    // second
    if (t.wSecond < 10)
        wsprintf(second, TEXT("0%i"), t.wSecond);
    else
        wsprintf(second, TEXT("%i"), t.wSecond);
    
    if (isUTC) {
        wsprintf(tempDate, TEXT("%i%s%sT%s%s%sZ"), t.wYear, month, day, hour, minute, second);
    } else {
        wsprintf(tempDate, TEXT("%i%s%sT%s%s%s"), t.wYear, month, day, hour, minute, second);
    }   

}


/*
* Is like doubleToSystemTime but set hour to 0. It used directly for anniversary and birthday.
* if not the time represented by date is the same in which the date is written 
* on the device
*/
void doubleToSystemTimeBirthday(wchar_t* tempDate, DATE date) {
        
    SYSTEMTIME t;
    
    VariantTimeToSystemTime(date, &t); 
    
    t.wHour = 0;
    t.wMinute = 0;
    t.wSecond = 0;

    wchar_t month [10];
    wchar_t day   [10];
    
    // month
    if (t.wMonth < 10)
        wsprintf(month, TEXT("0%i"), t.wMonth);
    else
        wsprintf(month, TEXT("%i"), t.wMonth);
    
    // day
    if (t.wDay  < 10)
        wsprintf(day, TEXT("0%i"), t.wDay);
    else
        wsprintf(day, TEXT("%i"), t.wDay);
    
    
    wsprintf(tempDate, TEXT("%i-%s-%s"), t.wYear, month, day);
    

}

/*
* The method transform a date in format "yyyy-MM-dd" 
*
* @param dataString : the string in "YYYYMMDDThhmmss" TEXT("YYYYMMDDThhmmssZ") format
* @param date       : the returned value into VariantTime format
* @action           : the parameter to set manually hour and minute. If NULL current value are used
*/
void systemTimeToDoubleBirthday(wchar_t* dataString, DATE* date) {

    wchar_t* token  = NULL;
    wchar_t* dummy  = NULL;
    wchar_t* Ztoken = NULL;
    
    wchar_t internalString[DIM_LOCAL_TEMP];
    wsprintf(internalString, dataString);
    wchar_t tmp[50];
    
    int ret = 0;
    
    SYSTEMTIME t;
    TIME_ZONE_INFORMATION tmz;        
    
    GetTimeZoneInformation(&tmz);
    
    Ztoken = wcschr(dataString, '-');
    if (Ztoken == NULL) {
        systemTimeToDouble(dataString, date, NULL);
        return;
    }
    
    //  TEXT("yyyy-MM-dd") 
    token = wcstok(internalString, TEXT("-"));

    if (token != NULL) {
            
        //yyyy
        wcsncpy(tmp, dataString, 4);
        tmp[4] = 0;
        swscanf(tmp, TEXT("%d"),&t.wYear);
        //MM
        wcsncpy(tmp, dataString + 5, 2);
        tmp[2] = 0;
        swscanf(tmp, TEXT("%d"),&t.wMonth);
        //dd
         wcsncpy(tmp, dataString + 8, 2);
        tmp[2] = 0;
        swscanf(tmp, TEXT("%d"),&t.wDay);
        
        t.wHour = 0;
        t.wMinute = 0;
        t.wSecond = 0;
                   
        // set to default
        t.wMilliseconds = 0;                    
        t.wDayOfWeek = 0;                       
        
        ret = SystemTimeToVariantTime (&t, date);
       
            
    } else {
        
        (*date) = 949998.000000;
        
    }

}



/*
* The method transform a date in format "YYYYMMDDThhmmss" or TEXT("YYYYMMDDThhmmssZ") with hour 0-24 into VariantTime format.
*
* @param dataString : the string in "YYYYMMDDThhmmss" TEXT("YYYYMMDDThhmmssZ") format
* @param date       : the returned value into VariantTime format
* @action           : the parameter to set manually hour and minute. If NULL current value are used
*/
void systemTimeToDouble(wchar_t* dataString, DATE* date, wchar_t* action) {

    wchar_t* token  = NULL;
    wchar_t* dummy  = NULL;
    wchar_t* Ztoken = NULL;
    BOOL isUTC      = FALSE;
    wchar_t internalString[DIM_LOCAL_TEMP];
    wsprintf(internalString, dataString);
    wchar_t tmp[50];
    
    int ret = 0;
    
    SYSTEMTIME t;
    TIME_ZONE_INFORMATION tmz;        
    
    GetTimeZoneInformation(&tmz);
    
    //  TEXT("20041006T203457") or TEXT("20041006T203457Z")
    token = wcstok(internalString, TEXT("T"));
    Ztoken = wcschr(dataString, 'Z');
    if (Ztoken != NULL) {
        isUTC = TRUE;
    }
    
    if (token != NULL) {
            
        //yyyy
        wcsncpy(tmp, dataString, 4);
        tmp[4] = 0;
        swscanf(tmp, TEXT("%d"),&t.wYear);
        //MM
        wcsncpy(tmp, dataString + 4, 2);
        tmp[2] = 0;
        swscanf(tmp, TEXT("%d"),&t.wMonth);
        //dd
         wcsncpy(tmp, dataString + 6, 2);
        tmp[2] = 0;
        swscanf(tmp, TEXT("%d"),&t.wDay);

        if (action != NULL && wcscmp(action, TEXT("0000")) == 0) {
            t.wHour = 0;
            t.wMinute = 0;
            t.wSecond = 0;
            
        } else if (action != NULL && wcscmp(action, TEXT("2359")) == 0) {
            t.wHour = 23;
            t.wMinute = 59;
            t.wSecond = 59;
        } else {
            if (wcslen(dataString) > 9) {
                //HH
                wcsncpy(tmp, dataString + 9, 2);
                tmp[2] = 0;
                swscanf(tmp, TEXT("%d"),&t.wHour);
                                
                //mm
                 wcsncpy(tmp, dataString + 11, 2);
                tmp[2] = 0;
                swscanf(tmp, TEXT("%d"),&t.wMinute);
                //ss
                 wcsncpy(tmp, dataString + 13, 2);
                tmp[2] = 0;
                swscanf(tmp, TEXT("%d"),&t.wSecond);
                

            } else {
                t.wHour = 0;
                t.wMinute = 0;
                t.wSecond = 0;
            }

        }
        // set to default
        t.wMilliseconds = 0;                    
        t.wDayOfWeek = 0;                       
                
        
        if(isUTC) {
            UTCToLocalTime(t);
        }
        ret = SystemTimeToVariantTime (&t, date);
       
            
    } else {
        
        (*date) = 949998.000000;
        
    }

}

/*
* return date and time of a date and if it is in utc or not
*/
BOOL getDateAndTimeAsIs(wchar_t* dataString, DATE* date) {

    wchar_t* token  = NULL;
    wchar_t* dummy  = NULL;
    wchar_t* Ztoken = NULL;
    BOOL isUTC      = FALSE;
    wchar_t internalString[DIM_LOCAL_TEMP];
    wsprintf(internalString, dataString);
    wchar_t tmp[50];
    
    int ret = 0;
    
    SYSTEMTIME t;
    TIME_ZONE_INFORMATION tmz;        
    
    GetTimeZoneInformation(&tmz);
    
    //  TEXT("20041006T203457") or TEXT("20041006T203457Z")
    token = wcstok(internalString, TEXT("T"));
    Ztoken = wcschr(dataString, 'Z');
    if (Ztoken != NULL) {
        isUTC = TRUE;
    }
    
    if (token != NULL) {
            
        //yyyy
        wcsncpy(tmp, dataString, 4);
        tmp[4] = 0;
        swscanf(tmp, TEXT("%d"),&t.wYear);
        //MM
        wcsncpy(tmp, dataString + 4, 2);
        tmp[2] = 0;
        swscanf(tmp, TEXT("%d"),&t.wMonth);
        //dd
         wcsncpy(tmp, dataString + 6, 2);
        tmp[2] = 0;
        swscanf(tmp, TEXT("%d"),&t.wDay);
        
        if (wcslen(dataString) > 9) {
            //HH
            wcsncpy(tmp, dataString + 9, 2);
            tmp[2] = 0;
            swscanf(tmp, TEXT("%d"),&t.wHour);
                            
            //mm
             wcsncpy(tmp, dataString + 11, 2);
            tmp[2] = 0;
            swscanf(tmp, TEXT("%d"),&t.wMinute);
            //ss
             wcsncpy(tmp, dataString + 13, 2);
            tmp[2] = 0;
            swscanf(tmp, TEXT("%d"),&t.wSecond);
            
        } else {
            t.wHour = 0;
            t.wMinute = 0;
            t.wSecond = 0;
        }

        
        // set to default
        t.wMilliseconds = 0;                    
        t.wDayOfWeek = 0;                       
                                
        ret = SystemTimeToVariantTime (&t, date);
       
            
    } else {
        
        (*date) = 949998.000000;
        
    }
    return isUTC;

}


/*
* The method returns the file name from the complete path. 
*
* @param path     : the complete path
* @param filename : the filename retrieved form path
*/

void getFileName(const wchar_t* path, wchar_t* filename) {
    wchar_t* p;

    p = wcsrchr(path, L'\\');

    if (p == NULL) {
        wcscpy(filename, path);
        return;
    }

    wcscpy(filename, p+1);
}

/*
* The method converts the path separator "\" with "@".
* Used for favorites items to permit to sync files and directory with a filesystem sync source.
* "\" char is not permitted in file name on windows machine
* This conversion happens on send items to server
*
* @param filename : the file name retrieved to be modified
*/

void convertAtSlash(wchar_t* filename) {
    
    int position = 0;
    string s = string();
    s += (wchar_t*)filename;

    position = pos(TEXT("@"), s);
    while (position != -1) {
        del(s, position, 1);
        ins(TEXT("\\"), s, position);
        position = pos(TEXT("@"), s);
    }        

    wsprintf(filename, (wchar_t*)s);
}

/*
* The method converts the separator "@" with "\".
* Used for favorites items to permit to sync files and directory with a filesystem sync source.
* "\" char is not permitted in file name on windows machine
* This conversion happens on returned items from server
*
* @param filename : the file name retrieved to be modified
*/

void convertSlashAt(wchar_t* filename) {
    
    int position = 0;
    string s = string();
    s += (wchar_t*)filename;

    // search for \;
    position = pos(TEXT("\\"), s);
    while (position != -1) {
        del(s, position, 1);
        ins(TEXT("@"), s, position);
        position = pos(TEXT("\\"), s);
    }
    

    wsprintf(filename, (wchar_t*)s);
}

/*
* The method get the complete directory name starting from path.
* 
* @param path     : the path and filename
* @param dirname  : the returned dir name
*/
void getDirName(const wchar_t* path, wchar_t* dirname) {

    string s = string();
    int position = 0;

    s += (wchar_t*) path;
    position = rpos(L'\\', s);
    
    if (position == -1) {
        wsprintf(dirname, path);
        return;
    }

    del(s, position, length(s) - position);
    if (dbg)
        MessageBox (NULL, (wchar_t*)s, TEXT ("1 in getDirName"), MB_SETFOREGROUND |MB_OK);
    
    getFileName((wchar_t*)s, dirname);
    
    if (dbg)
        MessageBox (NULL, dirname, TEXT ("dirName in getDirName"), MB_SETFOREGROUND |MB_OK);

}

/*
* return the length for the base64 array starting from length of the original array
*/
int lengthForB64(int len) {
    
    int modules = 0;
    int ret     = 0;

    modules = len % 3;
    if (modules == 0) {
        ret = 4 * (len / 3);

    } else {
        ret = 4 * ((len/3) + 1);

    }
    return ret;


}


/*
* The method encode in B64 the wchar_t array.
* 
* @param toEncode - the wchar_t array to encode
*/

wchar_t* encodeBase64W(wchar_t* toEncode) {

    char*    c        = NULL;
    char*    base64   = NULL;
    int      len      = 0, encodeLen = 0;  
    wchar_t* toReturn = NULL;

    c = wchar2char(toEncode);
    if (c == NULL) {
        return NULL;
    }
    
    len = strlen(c);
    encodeLen = lengthForB64(len);

    base64 = new char[encodeLen + 1];
    
    encodeBase64(c, base64, len);

    toReturn = char2wchar(base64);
    
    if (c != NULL)
        delete c;
    
    if (base64 != NULL)
        delete base64;
    
    return toReturn;    

}

/*
* The method decoded the wchar_t array.
* 
* @param toDecode - the wchar_t array to decode
*/

wchar_t* decodeBase64W(wchar_t* toDecode) {
    
    int      num        = 0;
    byte*    reversed   = NULL;
    char*    content    = NULL;
    wchar_t* toReturn   = NULL;
    int      len      = 0, encodeLen = 0;  

    content = wchar2char(toDecode);
    
    if (content == NULL) {        
        return NULL;
    }    
    len = strlen(content);
    // encodeLen = lengthForB64(len);
    
    // reversed = new byte[4 * strlen(content) + 1];   
    // memset(reversed, 0, 4 * strlen(content) + 1);

    reversed = new byte[len + 1];   
    memset(reversed, 0, len + 1);
    

    decodeBase64(content, reversed, &num);
    reversed[num] = 0;
        
    toReturn = char2wchar((char*)reversed);

    if (content != NULL)
        delete content; 
    
    if (reversed != NULL)
        delete reversed; 
    
    return toReturn;    

}

/*
* encode in base 64 starting from char* that was trasnformed into utf8
*/
wchar_t* encodeBase64UTF8(wchar_t* toEncode) {
    
    int len = 0;
    int k   = 0;
    char* messageXmlChar = NULL;
    wchar_t* t = NULL;
    char*    base64   = NULL;
    wchar_t* toReturn = NULL;

    len = wcslen(toEncode);            

    //
    // calculate lenght of array
    //
    k = WideCharToMultiByte (CP_UTF8, 0, toEncode, len, messageXmlChar, 0, NULL, NULL);

    if (k != 0) {
        messageXmlChar = new char[k + 1];

        k = WideCharToMultiByte (CP_UTF8, 0, toEncode, len, messageXmlChar, k, 0, 0);	
	    
	    messageXmlChar[k] = 0;                  
        
        len = strlen(messageXmlChar);

        k = lengthForB64(len);
        base64 = new char[k + 1];

        encodeBase64(messageXmlChar, base64, len);
        toReturn = char2wchar(base64);
       
    }

    if (messageXmlChar != NULL) {
        delete messageXmlChar;
    }

    if (base64 != NULL)
        delete base64;
    
    return toReturn;    

}

/*
* decoding from b64 and trasforming decoding from UTF8.
* used for contact, calendar, task
*/

wchar_t* decodeBase64WUTF8(wchar_t* toDecode) {
    
    int      num        = 0;
    byte*    reversed   = NULL;
    char*    content    = NULL;
    wchar_t* toReturn   = NULL;
    int k = 0;

    content = wchar2char(toDecode);
    
    if (content == NULL) {        
        return NULL;
    }    

    reversed = new byte[strlen(content) + 1];   
    memset(reversed, 0, strlen(content) + 1);
    
    decodeBase64(content, reversed, &num);
    reversed[num] = 0;    
    
    k = MultiByteToWideChar(CP_UTF8, 0, (char*)reversed, -1, NULL, 0);    
    //
    // MultiByteToWideChar returns the number of byte required including the terminator too
    //
    toReturn = new wchar_t[k];
    memset(toReturn, 0, k);

    MultiByteToWideChar(CP_UTF8, 0, (char*)reversed, num, toReturn, k);

    if (content != NULL)
        delete content; 
    
    if (reversed != NULL)
        delete reversed; 
    
    return toReturn;    

}


/*
* This function gets a unique file name in the current directory based on the parameter filename.
* It checks in the specified dir if fileName exists. If none it return the same file name. 
* If one arleady exists it return a file name like - file (1) - according to the 
* rules applied by windows for PPC
* 
* @param fileName : the file name that can be modified if the same file arleady exists
*/

void getUniqueFileName(wchar_t* fileName) {
        
    LOG.debug(TEXT("uniqueFileBegin"));
    BOOL exists = FALSE;
    FILE* f = NULL;
    int i = 1;

    f = _wfopen(fileName, TEXT("r"));

    if (f == NULL) {
        return;         
    }
    
    fflush(f);
    fclose(f);

    wchar_t path [DIM_LOCAL_TEMP];
    wchar_t file [DIM_LOCAL_TEMP];
    wsprintf(path, fileName);

    getFileName(fileName, file); // file contains file name
    
    getDirName(fileName, path); // path

    wchar_t* p = NULL;
    
    string s = string();
    int position = 0;
    s += (wchar_t*)file;
    position = pos(PWI_EXTENSION, s);
    if (position != -1) {
        file[position] = 0;
    }      
   
    do {
        wchar_t localFile[DIM_LOCAL_TEMP];
        wsprintf(localFile, TEXT("%s%s%s (%i)%s") , path, TEXT("\\"), file, i, PWI_EXTENSION);
        
        f = _wfopen(localFile, TEXT("r"));
    
        if (f == NULL) {
            wsprintf(fileName, localFile);
            break;              
        }    
        fflush(f);
        fclose(f);

        i++;
    } while(1);
    LOG.debug(TEXT("uniqueFileEnd"));
}



void UTCToLocalTime(SYSTEMTIME &sysTime)
{
     FILETIME LocalFileTime, SystemFileTime;
     SystemTimeToFileTime(&sysTime, &SystemFileTime);
     FileTimeToLocalFileTime(&SystemFileTime, &LocalFileTime);
     FileTimeToSystemTime(&LocalFileTime, &sysTime);
}

void localTimeToUTC(SYSTEMTIME &sysTime)
{
     FILETIME LocalFileTime, SystemFileTime;
     SystemTimeToFileTime(&sysTime, &LocalFileTime);
     LocalFileTimeToFileTime(&LocalFileTime, &SystemFileTime);
     FileTimeToSystemTime(&SystemFileTime, &sysTime);
}

void SafeGetTimeZoneInformation(TIME_ZONE_INFORMATION *ptzi) {

   ZeroMemory(ptzi, sizeof(TIME_ZONE_INFORMATION));

   // Ask the OS for the standard/daylight rules for the current time zone.
   if ((GetTimeZoneInformation(ptzi) == 0xFFFFFFFF) ||
       (ptzi->StandardDate.wMonth > 12) || (ptzi->DaylightDate.wMonth > 12))
   {
      // If the OS fails us, we default to the United States' rules.
      ZeroMemory(ptzi, sizeof(TIME_ZONE_INFORMATION));
      ptzi->StandardDate.wMonth =  10;  // October
      ptzi->StandardDate.wDay   =   5;  // Last Sunday (DOW == 0)
      ptzi->StandardDate.wHour  =   2;  // At 2:00 AM
      ptzi->DaylightBias        = -60;  // One hour difference
      ptzi->DaylightDate.wMonth =   4;  // April
      ptzi->DaylightDate.wDay   =   1;  // First Sunday (DOW == 0)
      ptzi->DaylightDate.wHour  =   2;  // At 2:00 AM
   }
}


time_t GetTransitionTimeT(TIME_ZONE_INFORMATION *ptzi, int year, BOOL fStartDST) {

   // We only handle years within the range that time_t supports.  We need to
   // handle the very end of 1969 since the local time could be up to 13 hours
   // into the previous year.  In this case, our code will actually return a
   // negative value, but it will be compared to another negative value and is
   // handled correctly.  The same goes for the 13 hours past a the max time_t
   // value of 0x7FFFFFFF (in the year 2038).  Again, these values are handled
   // correctly as well.

   if ((year < 1969) || (year > 2038)) {
      return (time_t)0;
   }

   SYSTEMTIME *pst = fStartDST ? &ptzi->DaylightDate : &ptzi->StandardDate;

   // WORD wYear          Year (0000 == 0)
   // WORD wMonth         Month (January == 1)
   // WORD wDayOfWeek     Day of week (Sunday == 0)
   // WORD wDay           Month day (1 - 31)
   // WORD wHour          Hour (0 - 23)
   // WORD wMinute        Minute (0 - 59)
   // WORD wSecond        Second (0 - 59)
   // WORD wMilliseconds  Milliseconds (0 - 999)

   // Compute the number of days since 1/1/1970 to the beginning of this year.
   long daysToYear = ((year - 1970) * 365) // Tally up previous years.
                   + ((year - 1969) >> 2); // Add few extra for the leap years.

   // Compute the number of days since the beginning of this year to the
   // beginning of the month.  We will add to this value to get the actual
   // year day.
   long yearDay = IS_LEAP_YEAR(year) ? M2LYD[pst->wMonth - 1] :
                                       M2YD [pst->wMonth - 1];

   // Check for day-in-month format.
   if (pst->wYear == 0) {

      // Compute the week day for the first day of the month (Sunday == 0).
      long monthDOW = (daysToYear + yearDay + BASE_DOW) % 7;

      // Add the day offset of the transition day to the year day.
      if (monthDOW < pst->wDayOfWeek) {
         yearDay += (pst->wDayOfWeek - monthDOW) + (pst->wDay - 1) * 7;
      } else {
         yearDay += (pst->wDayOfWeek - monthDOW) + pst->wDay * 7;
      }

      // It is possible that we overshot the month, especially if pst->wDay
      // is 5 (which means the last instance of the day in the month). Check
      // if the year-day has exceeded the month and adjust accordingly.
      if ((pst->wDay == 5) &&
          (yearDay >= (IS_LEAP_YEAR(year) ? M2LYD[pst->wMonth] :
                                            M2YD [pst->wMonth])))
      {
         yearDay -= 7;
      }

   // If not day-in-month format, then we assume an absolute date.
   } else {

      // Simply add the month day to the current year day.
      yearDay += pst->wDay - 1;
   }

   // Tally up all our days, hours, minutes, and seconds since 1970.
   long seconds = ((SECONDS_IN_A_DAY * (daysToYear + yearDay)) +
                   (3600L * (long)pst->wHour) +
                   (60L * (long)pst->wMinute) +
                   (long)pst->wSecond);

   // If we are checking for the end of DST, then we need to add the DST bias
   // since we are in DST when we chack this time stamp.
   if (!fStartDST) {
      seconds += ptzi->DaylightBias * 60L;
   }

   return (time_t)seconds;
}


BOOL IsDST(TIME_ZONE_INFORMATION *ptzi, time_t localTime) {

   // If either of the months is 0, then this usually means that the time zone
   // does not use DST.
   if ((ptzi->StandardDate.wMonth == 0) || (ptzi->DaylightDate.wMonth == 0)) {
      return FALSE;
   }

   // time_t   is a 32-bit value for the seconds since January 1, 1970
   // FILETIME is a 64-bit value for the number of 100-nanosecond intervals
   //          since January 1, 1601

   // Compute the FILETIME for the given local time.
   DWORDLONG dwl = ((DWORDLONG)116444736000000000 +
                   ((DWORDLONG)localTime * (DWORDLONG)10000000));
   FILETIME ft = *(FILETIME*)&dwl;

   // Convert the FILETIME to a SYSTEMTIME.
   SYSTEMTIME st;
   ZeroMemory(&st, sizeof(st));
   FileTimeToSystemTime(&ft, &st);

   // Get our start and end daylight savings times.
   time_t timeStart = GetTransitionTimeT(ptzi, (int)st.wYear, TRUE);
   time_t timeEnd   = GetTransitionTimeT(ptzi, (int)st.wYear, FALSE);

   // Check what hemisphere we are in.
   if (timeStart < timeEnd) {

      // Northern hemisphere ordering.
      return ((localTime >= timeStart) && (localTime < timeEnd));

   } else if (timeStart > timeEnd) {

      // Southern hemisphere ordering.
      return ((localTime < timeEnd) || (localTime >= timeStart));
   }

   // If timeStart equals timeEnd then this time zone does not support DST.
   return FALSE;
}



void UTCToLocalTime2(SYSTEMTIME &sysTime) {

    FILETIME LocalFileTime, SystemFileTime;
    LocalFileTime.dwLowDateTime = 0;
    LocalFileTime.dwHighDateTime = 0;
    SystemFileTime.dwLowDateTime = 0;
    SystemFileTime.dwHighDateTime = 0;

    SystemTimeToFileTime(&sysTime, &SystemFileTime);     
    FileTimeToLocalFileTime(&SystemFileTime, &LocalFileTime);     
    FileTimeToSystemTime(&LocalFileTime, &sysTime);

    wchar_t tmp[10];
    wsprintf(tmp, TEXT("The hour is UTCToLocalTime: %i"), sysTime.wHour );    
    LOG.error(tmp);

    // convert FILETIME to time_t
    FILETIME ftTmp = LocalFileTime;
    __int64 llTmp = 0;
    memcpy (&llTmp, &ftTmp, sizeof (__int64));
    llTmp = (llTmp - 116444736000000000) / 10000000;
    time_t t = (time_t) llTmp;

    TIME_ZONE_INFORMATION tzi;
    SafeGetTimeZoneInformation(&tzi);

    if (IsDST(&tzi, t))     // Is date t in daylight saving time?
    {
        // Subtract the number of minutes for the DST bias.  This is 
        // normally -60, meaning that we add 60 minutes.
        t -= (tzi.DaylightBias * 60);     
    }
    // Subtract the number of minutes for the StandardBias.
    t -= (tzi.StandardBias * 60);

    //t -= 3600;

    llTmp = t;
    llTmp = llTmp * 10000000 + 116444736000000000;
    memcpy (&ftTmp, &llTmp, sizeof (FILETIME));
    FileTimeToSystemTime(&ftTmp, &sysTime);
}

void localTimeToUTC2(SYSTEMTIME &sysTime)
{
     FILETIME LocalFileTime, SystemFileTime;
     SystemTimeToFileTime(&sysTime, &LocalFileTime);
     LocalFileTimeToFileTime(&LocalFileTime, &SystemFileTime);
     FileTimeToSystemTime(&SystemFileTime, &sysTime);
    
     wchar_t tmp[10];
     wsprintf(tmp, TEXT("The hour localTimeToUTC is: %i"), sysTime.wHour ); 
     LOG.error(tmp);
     wsprintf(tmp, TEXT("The hour localTimeToUTC is: %i"), sysTime.wMinute  ); 
     LOG.error(tmp);
     wsprintf(tmp, TEXT("The hour localTimeToUTC is: %i"), sysTime.wYear ); 
     LOG.error(tmp);
     wsprintf(tmp, TEXT("The hour localTimeToUTC is: %i"), sysTime.wMonth ); 
     LOG.error(tmp);
     wsprintf(tmp, TEXT("The hour localTimeToUTC is: %i"), sysTime.wDay  ); 
     LOG.error(tmp);

     // convert FILETIME to time_t
     FILETIME ftTmp = SystemFileTime;
     __int64 llTmp;
     memcpy (&llTmp, &ftTmp, sizeof (__int64));
     llTmp = (llTmp - 116444736000000000) / 10000000;
     time_t t = (time_t) llTmp;

     TIME_ZONE_INFORMATION tzi;
     SafeGetTimeZoneInformation(&tzi);
     
     if (IsDST(&tzi, t))     // Is date t in daylight saving time?
     {
        // Add the number of minutes for the DST bias.  This is 
        // normally -60, meaning that we subtract 60 minutes.
        t += (tzi.DaylightBias * 60);
     }
     // Add the number of minutes for the StandardBias.
     t += (tzi.StandardBias * 60);

     llTmp = t;
     llTmp = llTmp * 10000000 + 116444736000000000;
     memcpy (&ftTmp, &llTmp, sizeof (FILETIME));
     FileTimeToSystemTime(&ftTmp, &sysTime);
}
