/*
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//
// @author Marco Magistrali
//
#include <winbase.h>


#include <windows.h>
#include <oleauto.h>
#include "initguid.h"
#include "pimstore.h"
#include <stdio.h>
#include <time.h>

#include "common/Log.h"
#include "spds/common/Utils.h"
#include "spds/common/Constants.h"
#include "spds/common/SyncManagerFactory.h"
#include "pim/SettingFunctions.h"

#include "pim/LongObj.h"
#include "pim/ActionLog.h"
#include "pim/maincpp.h"
#include "pim/Utils.h"
#include "pim/ContactBuilder.h"
#include "pim/AppointmentBuilder.h"
#include "pim/TaskBuilder.h"
#include "pim/POOMClient2Server.h"
#include "pim/POOMServer2Client.h"
#include "pim/POOMFileManagement.h"
#include "pim/FILEClient2Server.h"
#include "pim/FILEServer2Client.h"


static  wchar_t CONTACTS    [ 9];
static  wchar_t APPOINTMENTS[13];       
static  wchar_t TASKS       [13];
static  wchar_t FAVORITES   [13];
static  wchar_t FILES       [13];
static  wchar_t NOTES       [13];
static  wchar_t ASSET       [13];

/*******************************/

/*
* Set all item for SLOW_SYNC. 
* It call the appropriate method to find POOM or FILE items
*
* @param source     : the source to complete with the items
* @param polApp     : the IPOutlookApp object. Used for POOM item (contact, calendar, task)
* @param dataType   : the constant value that represents the contact, calendar... data type
* @param path       : the install path of the SyncClient PIM application on the device
* @param dir        : the directory chose by user to sync briefcase files and notes
*/

void setAllItems(SyncSource &source, IPOutlookApp *polApp, int dataType, wchar_t* path, wchar_t* dir) {
    
    if (dataType == constFiles || 
        dataType == constNotes || 
        dataType == constFavorites)
        setAllItemsFILE(source, dataType, path, dir);
        
    
    //else if (dataType == constAsset)
    //    setAllItemsASSET(source);
    
    else 
        setAllItemsPOOM(source, polApp, dataType);
}

/*
* Set modified item for SLOW_SYNC. 
* It call the appropriate method to find new/update/delete POOM or FILE items
*
* @param source     : the source to complete with the items
* @param polApp     : the IPOutlookApp object. Used for POOM item (contact, calendar, task)
* @param dataType   : the constant value that represents the contact, calendar... data type
* @param path       : the install path of the SyncClient PIM application on the device
* @param dir        : the directory chose by user to sync briefcase files and notes
*/

void setModifiedItems(SyncSource &source, IPOutlookApp *polApp, int dataType, wchar_t* path, wchar_t* dir) {
    if (dataType == constFiles || 
            dataType == constNotes || 
            dataType == constFavorites)
        setModifiedItemsFILE(source, polApp, dataType, path, dir);
        
    //else if (dataType == constAsset)
    //    setModifiedItemsASSET(source, dataType, path, dir); 
    else
        setModifiedItemsPOOM(source, polApp, dataType, path);
}

/*
* Set modified item for SLOW_SYNC. 
* It call the appropriate method to find new/update/delete POOM or FILE items
*
* @param s          : the source to complete with the items
* @param polApp     : the IPOutlookApp object. Used for POOM item (contact, calendar, task)
* @param dataType   : the constant value that represents the contact, calendar... data type
* @param path       : the install path of the SyncClient PIM application on the device
* @param dir        : the directory chose by user to sync briefcase files and notes
*/

void manageReturnedItems(SyncSource& s, IPOutlookApp *polApp, int dataType, wchar_t* path, wchar_t* dir) {
    
    if (dataType == constFiles || 
        dataType == constNotes || 
        dataType == constFavorites)
        manageReturnedItemsFILE(s, dataType, path, dir);
        
    //else if (dataType == constAsset)
    //    manageReturnedItemsASSET(s, dataType, path, dir); 
    else
        manageReturnedItemsPOOM(s, polApp, dataType, path);

}

/*
* Used for display the mapping when a new item is added. It's for debug only
*
* @param s          : the source to complete with the items
*/

void displayMappings(SyncSource& s) {
    unsigned int n;
    SyncMap** map;

    n = s.getMapSize();
    map = s.getLUIDGUIDMapping();

    LOG.info(TEXT("Mappings"));
    LOG.info(TEXT("========"));
    for (unsigned int i=0; i<n; ++i) {
        wsprintf(logmsg, TEXT("luid: %s, guid: %s"), map[i]->getLUID(NULL), map[i]->getGUID(NULL));
        LOG.info(logmsg);
    }
}

/*
* Set modified item for SLOW_SYNC. 
* It call the appropriate method to find new/update/delete POOM or FILE items
*
* @param syncManager    : the sync manager that drives the sync process
* @param polApp         : the IPOutlookApp object. Used for POOM item (contact, calendar, task)
* @param sourceName     : the the name of the source to sync as ./contact, ./calendar...
* @param path           : the install path of the SyncClient PIM application on the device
* @param errorMessage   : the variable to store the message to return to vb program
* @param sync           : the sync mode as none, two-way
*/

int synchronize(SyncManager* syncManager, IPOutlookApp *polApp, wchar_t* sourceName, 
                wchar_t* path, wchar_t* errorMessage, wchar_t* dir, wchar_t* sync) {
    
    unsigned int ret = 0;

    // this int represents  the type folder object of pocketpcApplication
    int dataType = 0;           

    if (wcscmp(CONTACTS, sourceName) == 0) {        
        ActionLOG.action(SEPARATOR);
        ActionLOG.action(PREPARING_CONTACTS_SYNC);
        ActionLOG.action(SEPARATOR_BLANK);
        dataType = olFolderContacts ;
    }
    else if (wcscmp(APPOINTMENTS, sourceName) == 0){
        ActionLOG.action(SEPARATOR);
        ActionLOG.action(PREPARING_APPOINTMENTS_SYNC);
        ActionLOG.action(SEPARATOR_BLANK);
        dataType = olFolderCalendar;
    }    
    else if (wcscmp(TASKS, sourceName) == 0){
        ActionLOG.action(SEPARATOR);
        ActionLOG.action(PREPARING_TASKS_SYNC);
        ActionLOG.action(SEPARATOR_BLANK);
        dataType = olFolderTasks;
    }   
    else if (wcscmp(FAVORITES, sourceName) == 0){
        ActionLOG.action(SEPARATOR);
        ActionLOG.action(PREPARING_FAVORITES_SYNC);
        ActionLOG.action(SEPARATOR_BLANK);
        dataType = constFavorites;
    }  
    else if (wcscmp(FILES, sourceName) == 0){
        ActionLOG.action(SEPARATOR);
        ActionLOG.action(PREPARING_FILES_SYNC);
        ActionLOG.action(SEPARATOR_BLANK);
        dataType = constFiles;
    }  
    else if (wcscmp(NOTES, sourceName) == 0){
        ActionLOG.action(SEPARATOR);
        ActionLOG.action(PREPARING_NOTES_SYNC);
        ActionLOG.action(SEPARATOR_BLANK);
        dataType = constNotes;
    }  
    else if (wcscmp(ASSET, sourceName) == 0){
        ActionLOG.action(SEPARATOR);
        ActionLOG.action(PREPARING_ASSET_SYNC);
        ActionLOG.action(SEPARATOR_BLANK);
        dataType = constAsset;
    }  

    SyncSource source = SyncSource(sourceName);    
    
    // "none" is the literal value of SYNC_NONE code    

    if (wcscmp(sync, TEXT("none")) == 0) {
        ActionLOG.action(NO_SYNC);
        goto finally;
    }

    if (syncManager == NULL) {          
        error();
        error(errorMessage);
        return -1;        
    }

    //
    // Initializations
    //    
    
    LOG.info(TEXT("Preparing sync..."));
        
    // for decide what kind of sync to do...
    if (existsFile(dataType, path)) {
        // if file exists is TWO_WAY sync (default)                    
    }
    else {
        source.setPreferredSyncMode(SYNC_SLOW);
        
    }

    ret = syncManager->prepareSync(source);

    if (ret != 0) {
        switch (ret) {
            case ERR_PROTOCOL_ERROR:
                LOG.error(MSG_ERR_PROTOCOL_ERROR );
                wsprintf(errorMessage,MSG_ERR_PROTOCOL_ERROR );
                break;

            case ERR_AUTH_NOT_AUTHORIZED:
                LOG.error(MSG_ERR_AUTH_REQUIRED);
                wsprintf(errorMessage, MSG_ERR_AUTH_REQUIRED);
                break;
            
            case ERR_AUTH_REQUIRED:
                LOG.error(MSG_ERR_AUTH_REQUIRED);
                wsprintf(errorMessage, MSG_ERR_AUTH_REQUIRED);
                break;

            case ERR_AUTH_EXPIRED:
                LOG.error(MSG_ERR_AUTH_EXPIRED);
                wsprintf(errorMessage, MSG_ERR_AUTH_EXPIRED);
                break;

            case ERR_SRV_FAULT:
                LOG.error(MSG_ERR_SRV_FAULT);
                wsprintf(errorMessage, MSG_ERR_SRV_FAULT);
                break;

            case ERR_NOT_FOUND:
                wsprintf(logmsg, MSG_ERR_NOT_FOUND, source.getName(NULL, 0));
                LOG.error(logmsg);
                wsprintf(errorMessage, MSG_ERR_NOT_FOUND, source.getName(NULL, 0));
                break;
            case ERR_INTERNET_CONNECTION_MISSING:
                LOG.error(MSG_ERR_INTERNET_CONNECTION_MISSING);
                wsprintf(errorMessage, MSG_ERR_INTERNET_CONNECTION_MISSING);
                break;               

            default:
                error();
                error(errorMessage);
                break;
        }

        return ret;
    }
        
        
    switch (source.getSyncMode()) {
                
        case SYNC_SLOW:

            LOG.info(TEXT("Starting SLOW sync..."));
            ActionLOG.action(STARTING_SLOW_SYNC);                       
            setAllItems(source, polApp, dataType, path, dir);           
            break;

        case SYNC_TWO_WAY:                                      
                        
            LOG.info(TEXT("Starting TWO_WAY sync..."));
            ActionLOG.action(STARTING_STD_SYNC);
            setModifiedItems(source, polApp, dataType, path, dir);
            break;
                
        case SYNC_NONE:
            break;

        default:
            break;
    }

        if (source.getSyncMode() == SYNC_NONE) {
            ActionLOG.action(NO_SYNC);
            goto finally;
        }

    LOG.info(TEXT("Starting sync..."));
    ret = syncManager->sync(source);
        
    if (ret != 0) {
        error();
        error(errorMessage);
        if (ret == ERR_CONNECT)
            return ret;
        else
            return -1;
    }
    
    manageReturnedItems(source, polApp, dataType, path, dir);
    
    displayMappings(source);
    
    LOG.info(TEXT("Ending sync..."));
    ActionLOG.action(ENDING_SYNC);
    
    if (syncManager->endSync(source) != 0) {
        
        error();
        error(errorMessage);
        return -1;
    }
                
    
finally:

    SyncItem **p;
    p = source.getNewSyncItems();
    safeDelete((void**)p, source.getNewSyncItemsCount());
    if (p != NULL) {
        delete [] p;
    }
    p = source.getUpdatedSyncItems();
    safeDelete((void**)p, source.getUpdatedSyncItemsCount());
    if (p != NULL) {
        delete [] p;
    }
    p = source.getDeletedSyncItems();
    safeDelete((void**)p, source.getDeletedSyncItemsCount());
    if (p != NULL) {
        delete [] p;
    }
        p = source.getAllSyncItems();
    safeDelete((void**)p, source.getAllSyncItemsCount());
    if (p != NULL) {
        delete [] p;
    }

    //
    // Release mapping
    //
    SyncMap** sm;
    sm = source.getLUIDGUIDMapping();
    safeDelete((void**)sm, source.getMapSize());
        
    lastErrorCode = 0;

    return ret;
}

wchar_t* getElementContentModified(wchar_t* xml, wchar_t* tag, unsigned int* pos) {
    wchar_t* p1       = NULL;
    wchar_t* p2       = NULL;
    wchar_t* ret      = NULL;
    BOOL charFound    = FALSE;

    wchar_t openTag[50];
    wchar_t closeTag[50];

    unsigned int xmlLength = wcslen(xml);
    unsigned int l = wcslen(tag);

    if (pos != NULL) {
        *pos = 0;
    }

    wsprintf(openTag, TEXT("<%s>"), tag);
    wsprintf(closeTag, TEXT("</%s>"), tag);

    p1 = wcsstr(xml, openTag);

    if (p1 == NULL) { // tag can have attributes
        //
        // This is abcxyz
        //
        //goto finally;

        // if p1 is null I try to discover the next '>' char to close the tag. If does not exist 
        // return NULL
        
        // try to find "<tagName/>". If found it return null.
        wsprintf(openTag, TEXT("<%s/>"), tag);
        p1 = wcsstr(xml, openTag);

        if (p1 != NULL) {
            goto finally;
        }
        
        // try to find "<tagName"
        wsprintf(openTag, TEXT("<%s"), tag);
        p1 = wcsstr(xml, openTag);

        if (p1 == NULL) {
            goto finally;
        }
        
        p1 = p1 + l + 1;   // <body_ 

        for (unsigned int k = 0; k < xmlLength; k++) { // Suppose max length as the xml string
            p1 = p1 + 1;
            if (*p1 == 0) {
                goto finally;
            }
            else if (*p1 == '>') {
                charFound = TRUE;
                p1 = p1 + 1;
                break;
            }            
        }
        if (!charFound)
            goto finally;
        
        
    } else {  // tag doesn't have attribute. Original version

        p1 = p1+l+2;
        
    }
    
    if (*p1 == 0) {
        //
        // This is abc<tag>\0
        //
        goto finally;
    }

    p2 = wcsstr(p1, closeTag);

    if (p2 == NULL) {
        //
        // This is abc<tag>xyz\0
        //
        goto finally;
    }

    ret = new wchar_t[p2-p1+1];

    wcsncpy(ret, p1, p2-p1);
    ret[p2-p1] = 0;

    if (pos != NULL) {
        *pos = p2-xml+l+3;
    }
   
    finally:

    return ret;

}

void openLog(wchar_t* path) {
    
    wchar_t value[256];
    value [0] = 0;
    
    getClientConfigurationInternal (NULL, PROPERTY_SOURCEPROP_SPDM_LOG_LEVEL, value, FALSE);
    LOG.setLogPath(path); 
    if (wcscmp(value, TEXT("0")) == 0) {
        Log::debugEnabled = FALSE;    
        Log::infoEnabled = FALSE;    
    } else if (wcscmp(value, TEXT("2")) == 0) {
        Log::debugEnabled = TRUE;    
        Log::infoEnabled = TRUE;    
    } else { // default info
        Log::debugEnabled = FALSE;    
        Log::infoEnabled = TRUE;    
    }   
  
}


DWORD WINAPI synchronization (wchar_t* vbErrorMsg, wchar_t* path) {
  
    openLog(path);
    wchar_t errorMessage[1024];
    int rc          = 0;
    lastErrorCode   = 0;
                
    errorMessage[0] = 0;

    wchar_t source    [60];
    wchar_t dir      [256];      
    wchar_t sync      [60];
    dir[0]  = 0;
    sync[0] = 0;
    /*
    wsprintf(CONTACTS,      TEXT("./contact"   ));
    wsprintf(APPOINTMENTS,  TEXT("./calendar"  ));
    wsprintf(TASKS,         TEXT("./task"      ));
    wsprintf(FAVORITES,     TEXT("./favorite"  ));
    wsprintf(FILES,         TEXT("./briefcase"));
    wsprintf(NOTES,         TEXT("./note"      ));
    */
    wsprintf(CONTACTS,      CONTACTS_NAME    );
    wsprintf(APPOINTMENTS,  APPOINTMENTS_NAME);
    wsprintf(TASKS,         TASKS_NAME       );
    wsprintf(FAVORITES,     FAVORITES_NAME   );
    wsprintf(FILES,         FILES_NAME       );
    wsprintf(NOTES,         NOTES_NAME       );
    
    // Initialization and creation SyncManager
    SyncManagerFactory factory = SyncManagerFactory();
    SyncManager* syncManager = factory.getSyncManager(APPLICATION_URI);
        
    // Initialization and creation IPOutlookApp
    IPOutlookApp    *polApp; // outlook app
    
    HRESULT hr;     


    // for remember code only...
    // int constFolderCalendar  =  9; 
    // int constFolderContacts  = 10;
    // int constFolderTasks     = 13;

    //
    // custom values
    //
    // int constFiles       = 600;
    // int constNotes       = 601;
    // int constFavorites   = 602;
    

    // Initialize COM for Pocket Outlook
    if (FAILED (CoInitializeEx (NULL, 0))) return -2;

    // Get the application object
    hr = CoCreateInstance (CLSID_Application, NULL, CLSCTX_INPROC_SERVER, IID_IPOutlookApp, (LPVOID*)&polApp);
    if (FAILED(hr)) return -2;

    // Log on to Pocket Outlook
    hr = polApp->Logon (NULL);
    if (FAILED (hr)) return -2;
    
    if (FAILED(hr)) return -2;              

    Config* config = NULL;

    config = new Config(APPLICATION_URI);
    BOOL ret = config->readFromDM();

    int sourceCount = config->getSyncSourceCount();

    for (int i = 0; i < sourceCount; i++) {
    config->getSyncSourceConfig(i).getName(source);
    config->getSyncSourceConfig(i).getDir(dir);
    config->getSyncSourceConfig(i).getSync(sync);
    
    if (wcscmp(CONTACTS, source) == 0) {
                        
        if (dbg) {
            MessageBox (NULL, TEXT("In Contacts"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
            MessageBox (NULL, path, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
        }
        rc = synchronize(syncManager, polApp, CONTACTS, path, errorMessage, dir, sync);
                            
        if (dbg) {
            MessageBox (NULL, errorMessage, TEXT ("ErrorMessage"), MB_SETFOREGROUND |MB_OK);
            MessageBox (NULL, TEXT("End contacts sync"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
        }

    }
    else if (wcscmp(APPOINTMENTS, source) == 0){

        if (dbg)
            MessageBox (NULL, TEXT("In Appointments"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
    
        rc = synchronize(syncManager, polApp, APPOINTMENTS, path, errorMessage, dir, sync);

        if (dbg)
            MessageBox (NULL, TEXT("End appointmens sync"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);

    }
    else if (wcscmp(TASKS, source) == 0){

        if (dbg)
            MessageBox (NULL, TEXT("In Tasks"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
    
        rc = synchronize(syncManager, polApp, TASKS, path, errorMessage, dir, sync);

        if (dbg)
            MessageBox (NULL, TEXT("End tasks sync"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);

    }
    else if (wcscmp(FAVORITES, source) == 0){

        if (dbg)
            MessageBox (NULL, TEXT("In Favorites"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
    
        rc = synchronize(syncManager, polApp, FAVORITES, path, errorMessage, dir, sync);

        if (dbg)
            MessageBox (NULL, TEXT("End Favorites sync"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);

    }
    else if (wcscmp(FILES, source) == 0){

        if (dbg)
            MessageBox (NULL, TEXT("In File"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
    
        rc = synchronize(syncManager, polApp, FILES, path, errorMessage, dir, sync);

        if (dbg)
            MessageBox (NULL, TEXT("End File sync"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);

    }
    else if (wcscmp(NOTES, source) == 0){

        if (dbg)
            MessageBox (NULL, TEXT("In Note"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
    
        rc = synchronize(syncManager, polApp, NOTES, path, errorMessage, dir, sync);

        if (dbg)
            MessageBox (NULL, TEXT("End Note sync"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);

    }
    else {
        if (dbg)
            MessageBox (NULL, TEXT("Nothing to sync"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
    }
        
    if (dbg) {
        wchar_t t[10];
        wsprintf(t, TEXT("%i"), rc);
        MessageBox (NULL, t, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);

    }

    if (rc != 0)
        break;
    }

    wsprintf(vbErrorMsg, errorMessage);
    vbErrorMsg[wcslen(vbErrorMsg)] = L' ';
    
    // Log off and release the Application object
    polApp->Logoff ();
    polApp->Release ();


    delete syncManager;
    LOG.info(TEXT("Sync ended."));
    ActionLOG.action(SYNC_ENDED);
    
    return rc;
}

/*
* Shif the day of week (or a combination of days) for the recurrence
*/
int convertDayOfWeekMaskBackward(int dayOfWeekMaskSent) {   
    int dayOfWeekMask = 0;
    if ((dayOfWeekMaskSent % 2) != 0) {
        dayOfWeekMask = dayOfWeekMaskSent >> 1;    
        dayOfWeekMask += 64;
    }
    else {
        dayOfWeekMask = dayOfWeekMaskSent >> 1;
    }
    return dayOfWeekMask;
}
int convertDayOfWeekMaskForward(int dayOfWeekMaskSent) {   
    int dayOfWeekMask = 0;
    if (dayOfWeekMaskSent >= 64) {
        dayOfWeekMask = dayOfWeekMaskSent << 1;    
        dayOfWeekMask -= 127;
    }
    else {
        dayOfWeekMask = dayOfWeekMaskSent << 1;
    }
    return dayOfWeekMask;
}

#ifdef _WIN32_WCE
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPWSTR lpCmdLine, int nShowCmd ) {
#else
int main(int argc, char** argv) {
#endif
    
    IPOutlookApp *polApp;
    
    if (FAILED (CoInitializeEx (NULL, 0))) {
        return -2;
    }

    // Get the application object
    HRESULT hr = CoCreateInstance (CLSID_Application, NULL, CLSCTX_INPROC_SERVER, IID_IPOutlookApp, (LPVOID*)&polApp);
    if (FAILED(hr)) {
        return -2;
    }
    
    // Log on to Pocket Outlook
    hr = polApp->Logon (NULL);
    if (FAILED (hr)) return -2;    
    if (FAILED (hr)) return -2;              

    IFolder                 *pFolder;    
    IPOutlookItemCollection *pItems;
    IAppointment            *pAppointment;
    IAppointment            *pAppointmentNew;
    
    //
    // default folder calendar
    //
    polApp->GetDefaultFolder (9, &pFolder);

    pFolder->get_Items (&pItems);

    int countItems = 0, exCount = 0;
    long oid       = 0;
    pItems->get_Count(&countItems);

    pItems->Item (1, (IDispatch**)&pAppointment);    
    
    pAppointment->get_Oid(&oid);
  
    IRecurrencePattern  *pRecurrence;
    IRecurrencePattern  *pRecurrenceNew;
    BSTR                subject, location, categories, body,reminderSoundFile;
    DATE                dateStart, dateEnd, dateStartRec, timeStartRec, dateEndRec, timeEndRec;
    long                sensitivity, busyStatus, reminderOption, remonderMinutesBeforeStart ;
    VARIANT_BOOL                endDate;
    SYSTEMTIME          st;    
    
    long                elementLong = 0;
    long                recurrenceType = 0,
                        interval       = 0,
                        monthOfYear    = 0,
                        dayOfMonth     = 0,
                        dayOfWeekMask  = 0,
                        instance       = 0,
                        duration       = 0,
                        durationAppointment = 0;

    //
    // to get if a modification in the UTC goes into day before or after
    // store the initial value of day and check the last value
    //
    int x = 0, y = 0, z = 0;
    BOOL changeDay = FALSE, isBackward = FALSE;
    BOOL changeYearBackward = FALSE, changeYearForward = FALSE;

    // Convert Monday, 4/5/9 at 10:00 am to a DATE
    memset (&st, 0, sizeof(SYSTEMTIME));
    
    int ret = convertDayOfWeekMaskForward(32 + 16);

    pAppointment->get_Subject(&subject);   
	pAppointment->get_Location(&location);       	
	pAppointment->get_Categories(&categories);        
	pAppointment->get_ReminderSoundFile(&reminderSoundFile);        
	pAppointment->get_Body(&body);       

    pAppointment->get_Start(&dateStart);   
    pAppointment->get_Duration(&durationAppointment);
    pAppointment->get_End(&dateEnd);
   
    pAppointment->get_BusyStatus(&busyStatus);		
	pAppointment->get_Sensitivity(&sensitivity);		
	pAppointment->get_ReminderOptions(&reminderOption);		
	pAppointment->get_ReminderMinutesBeforeStart(&remonderMinutesBeforeStart);		
	
    VariantTimeToSystemTime(dateStart, &st); 

    //polApp->SystemTimeToVariantTime (&st, &date);
        
    // Get the recurrence pattern
    pAppointment->GetRecurrencePattern (&pRecurrence);
    
    pRecurrence->get_RecurrenceType(&recurrenceType);	
    pRecurrence->get_Interval(&interval);			
	pRecurrence->get_MonthOfYear(&monthOfYear);			
	pRecurrence->get_DayOfMonth(&dayOfMonth);		      	
	pRecurrence->get_Instance(&instance);
    pRecurrence->get_DayOfWeekMask(&dayOfWeekMask);

    pRecurrence->get_PatternStartDate(&dateStartRec);    
    pRecurrence->get_StartTime(&timeStartRec);
    pRecurrence->get_PatternEndDate(&dateEndRec);
    pRecurrence->get_NoEndDate(&endDate);

    pRecurrence->Release ();
    pAppointment->Release();

    pItems->Add((IDispatch**)&pAppointmentNew);    
    VariantTimeToSystemTime(dateStart, &st);
    x = st.wHour;
    localTimeToUTC(st);
    y = st.wHour;
    SystemTimeToVariantTime(&st, &dateStart); 
    pAppointmentNew->put_Start(dateStart);        
    pAppointmentNew->put_Duration(durationAppointment);

    VariantTimeToSystemTime(dateEnd, &st);
    localTimeToUTC(st);
    SystemTimeToVariantTime(&st, &dateEnd); 
    pAppointmentNew->put_End(dateEnd);        
    pAppointmentNew->put_Subject(TEXT("nuovoDaProg")); 
    pAppointmentNew->put_Location(location);       	
	pAppointmentNew->put_Categories(categories);        
	pAppointmentNew->put_ReminderSoundFile(reminderSoundFile);        
	pAppointmentNew->put_Body(body);       

    pAppointmentNew->put_BusyStatus(busyStatus);		
	pAppointmentNew->put_Sensitivity(sensitivity);		
	pAppointmentNew->put_ReminderOptions(reminderOption);		
	pAppointmentNew->put_ReminderMinutesBeforeStart(remonderMinutesBeforeStart);		
	    
    //
    // Tutte le modifiche presumono che il UTC va indietro di un giorno. Se non ci andasse
    // non devo fare la trasfomazione
    //
    pAppointmentNew->GetRecurrencePattern (&pRecurrenceNew);   
    
    

    TIME_ZONE_INFORMATION tmz;            
    int k = GetTimeZoneInformation(&tmz);        
    
    z = (x-y) < 0 ? (x-y)*(-1) : (x-y) ;    
    
    //
    // Change day?
    //
    changeDay = z > 12 ? TRUE : FALSE;
    if (changeDay) {
        if (tmz.Bias < 0)
            isBackward = TRUE; 
        else 
            isBackward = FALSE;
    }

    pRecurrenceNew->put_PatternStartDate(dateStartRec);   
    VariantTimeToSystemTime(dateEndRec, &st);
    localTimeToUTC(st);
    SystemTimeToVariantTime(&st, &dateEndRec); 
    pRecurrenceNew->put_PatternEndDate(dateEndRec);    
    
    pRecurrenceNew->put_NoEndDate(VARIANT_TRUE);

    pRecurrenceNew->put_RecurrenceType(recurrenceType);
    if (interval != 0)
        pRecurrenceNew->put_Interval(interval);			

    if (dayOfMonth != 0) {
        if (changeDay) {
            if (isBackward) {
                dayOfMonth = dayOfMonth - 1;
                if (dayOfMonth == 0) {
                    if (monthOfYear == 5 || monthOfYear == 7 || monthOfYear == 10) { 
                        // if May, July, October the month before has 30 days
                        dayOfMonth = 30;
                    } else {
                        dayOfMonth = 31;
                    }
                    changeYearBackward = TRUE;  
                }
            } else { // is 
                dayOfMonth = dayOfMonth + 1;
                if (dayOfMonth >  31  || 
                      (dayOfMonth == 31 && 
                        (monthOfYear == 4 || monthOfYear == 6 || monthOfYear == 9)
                       )
                    ) {
                    dayOfMonth = 1;
                    changeYearForward = TRUE;
                }
                

            }
        }
        pRecurrenceNew->put_DayOfMonth(dayOfMonth);		      	

    }
    if (monthOfYear != 0) {
        if (changeYearBackward) {
            monthOfYear = monthOfYear - 1;
            if (monthOfYear == 0) {
                monthOfYear = 12;
            }
        } else if (changeYearForward) {
            monthOfYear = monthOfYear + 1;
            if (monthOfYear > 12) {
                monthOfYear = 1;
            }
        }
        
        pRecurrenceNew->put_MonthOfYear(monthOfYear);			

    }
    if (dayOfWeekMask != 0) {
        if (changeDay) {
            if (isBackward) {
                dayOfWeekMask = convertDayOfWeekMaskBackward(dayOfWeekMask);
            } else {
                dayOfWeekMask = convertDayOfWeekMaskForward(dayOfWeekMask);
            }
        }
        pRecurrenceNew->put_DayOfWeekMask(dayOfWeekMask);	
    }
    if (instance != 0)
        pRecurrenceNew->put_Instance(instance);        

    pAppointmentNew->Save();
    pRecurrenceNew->Release();  
    pAppointmentNew->Release ();
    //pAppointment->GetRecurrencePattern (&pRecurrence);
    //pRecurrence->get_DayOfWeekMask(&elementLong);
   
    // Release objects
    polApp->Release ();
   
    // pAppt2->Release ();

    return 0;
}

//*****************************************************************++
// USATO PER LA MODIFICA: il 1 di ogni mese. Modifico il dayOfMOnth mettendolo
// indietro di un giorno. Se diventa == 0 allora lo pongo a 31.
//*****************************************************************++
/*

    pAppointment->get_Subject(&subject);   
	pAppointment->get_Location(&location);       	
	pAppointment->get_Categories(&categories);        
	pAppointment->get_ReminderSoundFile(&reminderSoundFile);        
	pAppointment->get_Body(&body);       

    pAppointment->get_Start(&dateStart);   
    pAppointment->get_Duration(&durationAppointment);
    pAppointment->get_End(&dateEnd);
   
    pAppointment->get_BusyStatus(&busyStatus);		
	pAppointment->get_Sensitivity(&sensitivity);		
	pAppointment->get_ReminderOptions(&reminderOption);		
	pAppointment->get_ReminderMinutesBeforeStart(&remonderMinutesBeforeStart);		
	
    VariantTimeToSystemTime(dateStart, &st); 

    //polApp->SystemTimeToVariantTime (&st, &date);
        
    // Get the recurrence pattern
    pAppointment->GetRecurrencePattern (&pRecurrence);
    
    pRecurrence->get_RecurrenceType(&recurrenceType);	
    pRecurrence->get_Interval(&interval);			
	pRecurrence->get_MonthOfYear(&monthOfYear);			
	pRecurrence->get_DayOfMonth(&dayOfMonth);		      	
	pRecurrence->get_Instance(&instance);
    pRecurrence->get_DayOfWeekMask(&dayOfWeekMask);

    pRecurrence->get_PatternStartDate(&dateStartRec);    
    pRecurrence->get_PatternEndDate(&dateEndRec);
    pRecurrence->get_NoEndDate(&endDate);

    pRecurrence->Release ();
    pAppointment->Release();
    
    //localTimeToUTC(st);
    //polApp->SystemTimeToVariantTime(&st, &date);    
    //pAppointment->put_Start(date);        
    //pAppointment->put_Duration(duration);    
    //pAppointment->Save();
    

    pItems->Add((IDispatch**)&pAppointmentNew);    
    VariantTimeToSystemTime(dateStart, &st);
    localTimeToUTC(st);
    SystemTimeToVariantTime(&st, &dateStart); 
    pAppointmentNew->put_Start(dateStart);        
    pAppointmentNew->put_Duration(durationAppointment);

    VariantTimeToSystemTime(dateEnd, &st);
    localTimeToUTC(st);
    SystemTimeToVariantTime(&st, &dateEnd); 
    pAppointmentNew->put_End(dateEnd);        
    pAppointmentNew->put_Subject(TEXT("nuovoDaProg")); 
    pAppointmentNew->put_Location(location);       	
	pAppointmentNew->put_Categories(categories);        
	pAppointmentNew->put_ReminderSoundFile(reminderSoundFile);        
	pAppointmentNew->put_Body(body);       

    pAppointmentNew->put_BusyStatus(busyStatus);		
	pAppointmentNew->put_Sensitivity(sensitivity);		
	pAppointmentNew->put_ReminderOptions(reminderOption);		
	pAppointmentNew->put_ReminderMinutesBeforeStart(remonderMinutesBeforeStart);		
	    
    //
    // Tutte le modifiche presumono che il UTC va indietro di un giorno. Se non ci andasse
    // non devo fare la trasfomazione
    //
    pAppointmentNew->GetRecurrencePattern (&pRecurrenceNew);    	
    pRecurrenceNew->put_RecurrenceType(recurrenceType);
    if (interval != 0)
        pRecurrenceNew->put_Interval(interval);			
	if (monthOfYear != 0)
        pRecurrenceNew->put_MonthOfYear(monthOfYear);			
    if (dayOfMonth != 0) {
        dayOfMonth = dayOfMonth - 1;
        if (dayOfMonth == 0)
            dayOfMonth = 31;
        pRecurrenceNew->put_DayOfMonth(dayOfMonth);		      	

    }
    if (dayOfWeekMask != 0) {
        dayOfWeekMask = convertDayOfWeekMask(dayOfWeekMask);
        pRecurrenceNew->put_DayOfWeekMask(dayOfWeekMask);	
    }
    if (instance != 0)
        pRecurrenceNew->put_Instance(instance);
    
    VariantTimeToSystemTime(dateStartRec, &st);
    localTimeToUTC(st);
    SystemTimeToVariantTime(&st, &dateStartRec); 
    pRecurrenceNew->put_PatternStartDate(dateStartRec);
    //pRecurrenceNew->put_StartTime(timeStartRec); //se metto la data sovrascrivo quella dell'appointment
    //pRecurrenceNew->put_Duration(duration);
    VariantTimeToSystemTime(dateEndRec, &st);
    localTimeToUTC(st);
    SystemTimeToVariantTime(&st, &dateEndRec); 
    pRecurrenceNew->put_PatternEndDate(dateEndRec);
    //pRecurrenceNew->put_EndTime(timeEndRec); //se metto la data sovrascrivo quella dell'appointment
    
    pRecurrenceNew->put_NoEndDate(VARIANT_TRUE);

    pAppointmentNew->Save();
    pRecurrenceNew->Release();  
    pAppointmentNew->Release ();
    //pAppointment->GetRecurrencePattern (&pRecurrence);
    //pRecurrence->get_DayOfWeekMask(&elementLong);

   
    // Release objects
    polApp->Release ();

//





//*****************************************************************++
// USATO PER LA MODIFICA DEL PRIMO VENERDI DEL MESE. FUNZIONA MODIFICANDO DA VENERDI A GIOVEDI
//*****************************************************************++
/*
    pAppointment->get_Subject(&subject);   
	pAppointment->get_Location(&location);       	
	pAppointment->get_Categories(&categories);        
	pAppointment->get_ReminderSoundFile(&reminderSoundFile);        
	pAppointment->get_Body(&body);       

    pAppointment->get_Start(&dateStart);   
    pAppointment->get_Duration(&durationAppointment);
    pAppointment->get_End(&dateEnd);
   
    pAppointment->get_BusyStatus(&busyStatus);		
	pAppointment->get_Sensitivity(&sensitivity);		
	pAppointment->get_ReminderOptions(&reminderOption);		
	pAppointment->get_ReminderMinutesBeforeStart(&remonderMinutesBeforeStart);		
	
    VariantTimeToSystemTime(dateStart, &st); 

    //polApp->SystemTimeToVariantTime (&st, &date);
        
    // Get the recurrence pattern
    pAppointment->GetRecurrencePattern (&pRecurrence);
    
    pRecurrence->get_RecurrenceType(&recurrenceType);	
    pRecurrence->get_Interval(&interval);			
	pRecurrence->get_MonthOfYear(&monthOfYear);			
	pRecurrence->get_DayOfMonth(&dayOfMonth);		      	
	pRecurrence->get_Instance(&instance);
    pRecurrence->get_DayOfWeekMask(&dayOfWeekMask);

    pRecurrence->get_PatternStartDate(&dateStartRec);    
    pRecurrence->get_PatternEndDate(&dateEndRec);
    pRecurrence->get_NoEndDate(&endDate);

   
    pRecurrence->Release ();
    pAppointment->Release();
    
    //localTimeToUTC(st);
    //polApp->SystemTimeToVariantTime(&st, &date);    
    //pAppointment->put_Start(date);        
    //pAppointment->put_Duration(duration);    
    //pAppointment->Save();
    

    pItems->Add((IDispatch**)&pAppointmentNew);    
    VariantTimeToSystemTime(dateStart, &st);
    localTimeToUTC(st);
    SystemTimeToVariantTime(&st, &dateStart); 
    pAppointmentNew->put_Start(dateStart);        
    pAppointmentNew->put_Duration(durationAppointment);

    VariantTimeToSystemTime(dateEnd, &st);
    localTimeToUTC(st);
    SystemTimeToVariantTime(&st, &dateEnd); 
    pAppointmentNew->put_End(dateEnd);        
    pAppointmentNew->put_Subject(TEXT("nuovoDaProg")); 
    pAppointmentNew->put_Location(location);       	
	pAppointmentNew->put_Categories(categories);        
	pAppointmentNew->put_ReminderSoundFile(reminderSoundFile);        
	pAppointmentNew->put_Body(body);       

    pAppointmentNew->put_BusyStatus(busyStatus);		
	pAppointmentNew->put_Sensitivity(sensitivity);		
	pAppointmentNew->put_ReminderOptions(reminderOption);		
	pAppointmentNew->put_ReminderMinutesBeforeStart(remonderMinutesBeforeStart);		
	
    //pAppointmentNew->Save();

    pAppointmentNew->GetRecurrencePattern (&pRecurrenceNew);    	

    pRecurrenceNew->put_RecurrenceType(recurrenceType);
    if (interval != 0)
        pRecurrenceNew->put_Interval(interval);			
	if (monthOfYear != 0)
        pRecurrenceNew->put_MonthOfYear(monthOfYear);			
	if (dayOfMonth != 0)
        pRecurrenceNew->put_DayOfMonth(dayOfMonth);		      	
    if (dayOfWeekMask != 0) {
        dayOfWeekMask = convertDayOfWeekMask(dayOfWeekMask);
        pRecurrenceNew->put_DayOfWeekMask(dayOfWeekMask);	
    }
    if (instance != 0)
        pRecurrenceNew->put_Instance(instance);
    
    VariantTimeToSystemTime(dateStartRec, &st);
    localTimeToUTC(st);
    SystemTimeToVariantTime(&st, &dateStartRec); 
    pRecurrenceNew->put_PatternStartDate(dateStartRec);
    //pRecurrenceNew->put_StartTime(timeStartRec); //se metto la data sovrascrivo quella dell'appointment
    //pRecurrenceNew->put_Duration(duration);
    VariantTimeToSystemTime(dateEndRec, &st);
    localTimeToUTC(st);
    SystemTimeToVariantTime(&st, &dateEndRec); 
    pRecurrenceNew->put_PatternEndDate(dateEndRec);
    //pRecurrenceNew->put_EndTime(timeEndRec); //se metto la data sovrascrivo quella dell'appointment
    
    pRecurrenceNew->put_NoEndDate(VARIANT_TRUE);
   
    // Start Date
    //pRecurrence->get_PatternStartDate(&date);    
    //pRecurrence->get_DayOfWeekMask(&elementLong);
    //pRecurrence->put_PatternStartDate(date);
    //pRecurrenceNew->Release ();
           
    pAppointmentNew->Save();
    pRecurrenceNew->Release();  
    pAppointmentNew->Release ();
*/
//*****************************************************************++
//*****************************************************************++
//*****************************************************************++