/*
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//
// @author Marco Magistrali
//

#ifndef INCL_SETTINGSFUNCTIONS
    #define INCL_SETTINGSFUNCTIONS    
   
    #define PROPERTY_SOURCEPROP_SPDM_REMOTE_SETTINGS   TEXT("spdm.showRemoteSettings") 
    #define PROPERTY_SOURCEPROP_REMOTE_SETTINGS        TEXT("showRemoteSettings") 

    //used in Utils to determine if SystemTimeUTC or LocaTime conversion
    // 0 no conversion applied
    // 1 conversion applied

    //#define PROPERTY_SOURCEPROP_SPDM_SYSTEMTIME_TO_LOCALTIME   TEXT("spdm.adjustSystemTimeToLocalTime") 
    //#define PROPERTY_SOURCEPROP_SYSTEMTIME_TO_LOCALTIME        TEXT("adjustSystemTimeToLocalTime") 
    
    #define PROPERTY_SOURCEPROP_SPDM_LOG_LEVEL   TEXT("spdm.logLevel") 
    #define PROPERTY_SOURCEPROP_LOG_LEVEL        TEXT("logLevel") 


    #define PROPERTY_SOURCEPROP_SPDM_SHOW   TEXT("spdm.show") 
    #define PROPERTY_SOURCEPROP_SHOW        TEXT("show") 
    
    #define PROPERTY_SOURCE_SPDM_SYNC        TEXT("spdm.sync"     )
    #define PROPERTY_SOURCE_SPDM_DIR         TEXT("spdm.dir"      )
    #define PROPERTY_SOURCE_SPDM_URI         TEXT("spdm.uri"      )
    #define PROPERTY_SOURCE_SPDM_SYNC_MODES  TEXT("spdm.syncModes")

    #define CONTEXT_SPDS_SOURCESPROP TEXT("spds/sourcesprop")



    /*------------------ methods -----------------*/
    
    /*
    * Get the unique Device Identifier from PPC 2002 - 2003
    * see more at http://www.pocketpcdn.com/articles/serial_number2002.html
    *
    * @param deviceId     : the retrieved device id
    */

    void GetSerialNumberFromKernelIoControl(wchar_t* deviceId);

    /* 
    * Method to write communication settings into device. 
    * Used by setConfigurationParameters 
    *
    * @param rootContext  : the root context node
    * @param username     : user name
    * @param password     : user password
    * @param syncURL      : synchronization url
    * @param serverName   : the server uri used to address the sync server
    * @param deviceId     : device identifier
    * @param errorMsg     : the returned message to vb interface
    */

    void saveSyncMLPropsToDM(wchar_t* rootContext, wchar_t* username, wchar_t* password, 
                         wchar_t* syncURL, wchar_t* serverName, wchar_t* deviceId, wchar_t* errorMsg);

    /* 
    * Method to get communication settings from device. 
    * @param name  : name of the property to retrieve value of
    * @param value : value of the property
    */

    DWORD WINAPI getConfigParameter (wchar_t* name, wchar_t* value);

    /* 
    * Method to set communication settings into device. The parameters are retrieved by Visual Basic interface
    *
    * @param username     : user name
    * @param password     : user password
    * @param syncURL      : synchronization url
    * @param serverName   : the server uri used to address the sync server
    * @param errorMsg     : the returned message to vb interface
    */

    DWORD WINAPI setConfigParameters(wchar_t *username, wchar_t* password, 
                                 wchar_t* syncURL, wchar_t* serverName, wchar_t* errorMsg);



    /*
    * Method to get the synchronization setting parameters.
    * 
    * @param sourceName     : source name of which get parameter's value
    * @param propertyName   : name of property
    * @param propertyValue  : value of property
    */

    DWORD WINAPI getSourceParameter (LPWSTR sourceName, LPWSTR propertyName,  LPWSTR propertyValue);



    /*
    * Method to set the synchronization setting parameters.
    * 
    * @param sourceName     : source name of which get parameter's value
    * @param propertyName   : name of property
    * @param propertyValue  : value of property
    * @param errorMsg       : returned error message
    */

    DWORD WINAPI setSourceParameters (wchar_t* sourceName, wchar_t* propertyName, 
                                      wchar_t* propertyValue, wchar_t* errorMsg);


    /*
    * Method to get the client configurations. These property are set by installing process.
    * These are extra parameters not used in synchronization process 
    * but are needed for example to show or hide some fields in GUI interface.
    * 
    * @param sourceName     : source name of which get parameter's value
    * @param propertyName   : name of property
    * @param propertyValue  : value of property
    */

    DWORD WINAPI getClientConfiguration  (wchar_t* sourceName, wchar_t* propertyName, 
                                            wchar_t* propertyValue);
    
    DWORD getClientConfigurationInternal (wchar_t* sourceName, wchar_t* propertyName, 
                                            wchar_t* propertyValue, BOOL isFromVB);

    
#endif