/*
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//
// @author Marco Magistrali
//

#ifndef INCL_ASSET
    #define INCL_ASSET

    #include "common/fscapi.h"
    #include "pim/asset/AssetVersion.h"

// --------------------------------------------------------------- Constants
    
    // These are the possible state of the Asset
    #define STATE_NEW                           TEXT("N")
    #define STATE_UPDATE                        TEXT("U")
    #define STATE_DELETE                        TEXT("D")
    #define STATE_ASSET_INSTALLED               TEXT("AI")
    #define STATE_FILE_DOWNLOAD                 TEXT("FD")
    #define STATE_FILE_EXTRACTED                TEXT("FE")
    #define STATE_PREVIOUS_VERSION_UNINSTALLED  TEXT("PVU")
    #define STATE_NEW_VERSION_NOT_WANTED        TEXT("NVNW")
    #define STATE_NOT_VALID                     TEXT("NV")


    // The following constants are used for create an XML from an Asset
    // and for to represent a Asset into XML
    #define PROPERTY_ID                         TEXT("id"          )
    #define PROPERTY_NAME                       TEXT("name"        )
    #define PROPERTY_MANUFACTURER               TEXT("manufacturer")
    #define PROPERTY_DESCRIPTION                TEXT("description" )
    #define PROPERTY_STATE                      TEXT("state"       )
    #define PROPERTY_LAST_UPDATE                TEXT("last_update" )

    #define DIM_ASSET_VARIABLE                  256

    class /*__declspec(dllexport)*/ Asset {

    private:
        
        wchar_t id              [DIM_ASSET_VARIABLE];
        wchar_t name            [DIM_ASSET_VARIABLE];
        wchar_t manufacturer    [DIM_ASSET_VARIABLE];
        wchar_t description     [DIM_ASSET_VARIABLE];
        wchar_t state           [DIM_ASSET_VARIABLE];
        
        /*
        * Contains the current version of the asset. It always exists.
        */
        AssetVersion* currentVersion;
        
        /*
        * Contains the current version of the asset. It cannot exists if 
        * the asset do not has to be updated or installed
        */
        AssetVersion* newVersion;

    public:
        
        wchar_t* getId();
        void setId(wchar_t* u);

        wchar_t* getName();
        void setName(wchar_t* u);

        wchar_t* getManufacturer();
        void setManufacturer(wchar_t* u);

        wchar_t* getDescription();
        void setDescription(wchar_t* u);

        wchar_t* getState();
        void setState(wchar_t* u);

        AssetVersion* getCurrentVersion();
        void setCurrentVersion(AssetVersion* u);

        AssetVersion* getNewVersion();
        void setNewVersion(AssetVersion* u);

   };

#endif